option(ARMARX_CMAKE_ADDON_SLICE_PYTHON_GENERATION ON)

function(armarx_add_ice_library TARGET)
    # Parse arguments.
    set(multi_param SLICE_FILES DEPENDENCIES DEPENDENCIES_LEGACY)
    set(single_param)
    set(flag_param)
    cmake_parse_arguments(PARSE_ARGV 1 AX "${flag_param}" "${single_param}" "${multi_param}")
    if(DEFINED AX_UNPARSED_ARGUMENTS)
        message(FATAL_ERROR "${TARGET}: Unknown arguments `${AX_UNPARSED_ARGUMENTS}`.")
    endif()

    if(NOT DEFINED Ice_SLICE2CPP_EXECUTABLE)
        message(FATAL_ERROR "Ice_SLICE2CPP_EXECUTABLE is undefined!")
    endif()

    # Define variables for generic implementation
    list(APPEND SLICE_INCLUDE_DIRECTORIES ${Ice_SLICE_DIR} "${PROJECT_SOURCE_DIR}/source" "${PROJECT_BINARY_DIR}/source")
    set(SLICE2CPP_INPUT_DIR "${CMAKE_CURRENT_SOURCE_DIR}")
    set(SLICE2CPP_OUTPUT_DIR "${CMAKE_CURRENT_BINARY_DIR}")
    set(INPUT_INCLUDE_PREFIX "${PROJECT_BINARY_DIR}/source")
    set(SLICE2PY_OUTPUT_DIR "${CMAKE_BINARY_DIR}/python")

    if(${ARMARX_CMAKE_ADDON_SLICE_PYTHON_GENERATION})
        file(MAKE_DIRECTORY ${SLICE2PY_OUTPUT_DIR})
    endif()

    # General flags for slice2cpp.
    list(TRANSFORM SLICE_INCLUDE_DIRECTORIES PREPEND "-I")
    list(APPEND SLICE_COMMON_FLAGS
        ${SLICE_INCLUDE_DIRECTORIES}
        --underscore)

    # The uppercase string is needed for the --dll-export slice flag.
    string(TOUPPER ${TARGET} TARGET_UPPERCASE)
    string(REPLACE "-" "_" TARGET_UPPERCASE "${TARGET_UPPERCASE}")
    set(SLICE2CPP_FLAGS --dll-export ${TARGET_UPPERCASE}_IMPORT_EXPORT)

    # Project interface dependency paths go through all dependent projects.
    set(SLICE2CPP_INCLUDE_DIRECTORIES "")
    foreach(DEP ${AX_DEPENDENCIES})
        if (TARGET ${DEP})
            get_target_property(DEP_INCLUDES "${DEP}" SLICE2CPP_INCLUDE_DIRECTORIES)

            if(DEP_INCLUDES)
                set(SLICE2CPP_INCLUDE_DIRECTORIES ${DEP_INCLUDES})
            # LEGACY MODE - REMOVE ELSE()-PART AFTER MIGRATING ARMARXCORE.
            else()
                string(REPLACE "Interfaces" "" DEP "${DEP}")
                # append all directories found in <DEP>_INTERFACE_DIRS
                # as include directories for the current compilation process
                foreach(DEPENDEND_INTERFACE_DIR ${${DEP}_INTERFACE_DIRS})
                    list(APPEND SLICE2CPP_INCLUDE_DIRECTORIES "${DEPENDEND_INTERFACE_DIR}")
                endforeach()

                #include_directories(${${DEP}_INCLUDE_DIRS})
                # append all dependent interface libraries to the DEPENDEND_INTERFACE_LIBRARIES
                # CMake variable which is used further down in target_link_libraries()
                list(APPEND DEPENDEND_INTERFACE_LIBRARIES "${${DEP}_INTERFACE_LIBRARY}")
            endif()
        else ()
            message(STATUS "Dependency ${DEP} not found")
        endif ()
        # /LEGACY MODE
    endforeach()
    foreach(SLICE2CPP_INCLUDE_DIR ${SLICE2CPP_INCLUDE_DIRECTORIES})
        list(APPEND SLICE_COMMON_FLAGS "-I${SLICE2CPP_INCLUDE_DIR}")
    endforeach()

    # setup C++ files in the following loop
    set(LIB_FILES "")
    set(LIB_HEADER_FILES "")

    # Run slice2cpp on each Ice file.
    foreach(CURRENT_SLICE_FILE ${AX_SLICE_FILES})
        # Full path without extension ("path/file.ext" => "path/file").
        get_filename_component(CURRENT_SLICE_FILE_DIR "${CURRENT_SLICE_FILE}" DIRECTORY)
        get_filename_component(CURRENT_SLICE_FILE_NAME_WE "${CURRENT_SLICE_FILE}" NAME_WLE)
        if ("${CURRENT_SLICE_FILE_DIR}" STREQUAL "")
            set(STRIPPED_SLICE_FILE "${CURRENT_SLICE_FILE_NAME_WE}")
        else()
            set(STRIPPED_SLICE_FILE "${CURRENT_SLICE_FILE_DIR}/${CURRENT_SLICE_FILE_NAME_WE}")
        endif()

        # Absolute directory of current slice file
        set(CURRENT_SLICE_FILE_ABS_DIR "${CURRENT_SLICE_FILE_DIR}")
        if(NOT IS_ABSOLUTE "${CURRENT_SLICE_FILE_DIR}")
            if("${CURRENT_SLICE_FILE_DIR}" STREQUAL "")
                set(CURRENT_SLICE_FILE_ABS_DIR "${SLICE2CPP_OUTPUT_DIR}")
            else()
                set(CURRENT_SLICE_FILE_ABS_DIR "${SLICE2CPP_OUTPUT_DIR}/${CURRENT_SLICE_FILE_DIR}")
            endif()
        endif()

        # Make sure the directory for generated files exists.
        file(MAKE_DIRECTORY ${CURRENT_SLICE_FILE_ABS_DIR})

        # put generated files in a directory with the same hierarchy and names as the .ice file
        set(SLICE_CURRENT_CPP_FLAGS "")
        list(APPEND SLICE_CURRENT_CPP_FLAGS ${SLICE_COMMON_FLAGS} --output-dir ${CURRENT_SLICE_FILE_ABS_DIR})

        # Always add a prefix to the #include directives of generated files
        # Since only #include <> directives are generated the local directory is not
        # regarded when searching for matching header files which would require
        # adding additional include_directories.
        string(REGEX REPLACE "^${INPUT_INCLUDE_PREFIX}/" "" SLICE_FILE_INCLUDE_DIR "${CURRENT_SLICE_FILE_ABS_DIR}")
        list(APPEND SLICE_CURRENT_CPP_FLAGS --include-dir "${SLICE_FILE_INCLUDE_DIR}")

        # Handle absolute and relative paths to get absolute input and output file paths
        if(IS_ABSOLUTE "${CURRENT_SLICE_FILE}")
            set(SLICE2CPP_INPUT_FILE "${CURRENT_SLICE_FILE}")
        else()
            set(SLICE2CPP_INPUT_FILE "${SLICE2CPP_INPUT_DIR}/${CURRENT_SLICE_FILE}")
        endif()
        if(IS_ABSOLUTE "${STRIPPED_SLICE_FILE}")
            set(SLICE2CPP_OUTPUT_FILE "${STRIPPED_SLICE_FILE}")
        else()
            set(SLICE2CPP_OUTPUT_FILE "${SLICE2CPP_OUTPUT_DIR}/${STRIPPED_SLICE_FILE}")
        endif()


        # Location for depend files
        # The depend files are put into the same directory as the *.ice file
        if (IS_ABSOLUTE "${CURRENT_SLICE_FILE}")
            set(CPP_DEPEND_FILE "${CURRENT_SLICE_FILE_DIR}/slice_cpp_depend_${CURRENT_SLICE_FILE_NAME_WE}.txt")
            set(ESCAPED_DIR "${CURRENT_SLICE_FILE_DIR}/")
        else()
            if ("${CURRENT_SLICE_FILE_DIR}" STREQUAL "")
                set(STRIPPED_SLICE_FILE "${CURRENT_SLICE_FILE_NAME_WE}")
                set(CPP_DEPEND_FILE "${CMAKE_CURRENT_BINARY_DIR}/slice_cpp_depend_${CURRENT_SLICE_FILE_NAME_WE}.txt")
                set(ESCAPED_DIR "${CURRENT_SLICE_FILE_DIR}")
            else()
                set(CPP_DEPEND_FILE "${CMAKE_CURRENT_BINARY_DIR}/${CURRENT_SLICE_FILE_DIR}/slice_cpp_depend_${CURRENT_SLICE_FILE_NAME_WE}.txt")
                set(ESCAPED_DIR "${CURRENT_SLICE_FILE_DIR}/")
            endif()
        endif()

        get_filename_component(CPP_DEPEND_FILE_PARENT ${CPP_DEPEND_FILE} DIRECTORY)
        file(MAKE_DIRECTORY "${CPP_DEPEND_FILE_PARENT}")


        # Generate the header and implementation files from the slice file.
        # The slice2cpp outputs the depend file in a slightly incorrect format: The file of which the
        # dependencies are listed is given as the filename only without any relative path in front.
        # The sed command turns the filename into the correct relative path which is relative to
        # CMAKE_CURRENT_BINARY_DIR.
        add_custom_command(
           OUTPUT "${SLICE2CPP_OUTPUT_FILE}.h"
                  "${SLICE2CPP_OUTPUT_FILE}.cpp"
           COMMAND "${Ice_SLICE2CPP_EXECUTABLE}" ${SLICE_CURRENT_CPP_FLAGS} ${SLICE2CPP_FLAGS} "${SLICE2CPP_INPUT_FILE}" --depend --depend-file ${CPP_DEPEND_FILE}
           COMMAND /bin/sed -i -e '1s@^@${ESCAPED_DIR}@' "${CPP_DEPEND_FILE}"
           COMMAND "${Ice_SLICE2CPP_EXECUTABLE}" ${SLICE_CURRENT_CPP_FLAGS} ${SLICE2CPP_FLAGS} "${SLICE2CPP_INPUT_FILE}"
           DEPENDS "${Ice_SLICE2CPP_EXECUTABLE}"
           DEPFILE "${CPP_DEPEND_FILE}"
           BYPRODUCTS "${CPP_DEPEND_FILE}"
           MAIN_DEPENDENCY "${SLICE2CPP_INPUT_FILE}"
           COMMENT "Generating `${SLICE2CPP_OUTPUT_FILE}.{h|cpp}` from `${SLICE2CPP_INPUT_FILE}`."
        )

        # Generate python code
        # TODO: This does only run at cmake generation time - is this the desired behavior?
        if(${ARMARX_CMAKE_ADDON_SLICE_PYTHON_GENERATION})
            set(SLICE2PY_ARGS ${SLICE_COMMON_FLAGS})
            list(APPEND SLICE2PY_ARGS "--output-dir=${SLICE2PY_OUTPUT_DIR}")

            execute_process(
                COMMAND "${Ice_SLICE2PY_EXECUTABLE}" ${SLICE2PY_ARGS} "${SLICE2CPP_INPUT_FILE}"
                INPUT_FILE ${SLICE2CPP_INPUT_FILE}
                RESULT_VARIABLE SLICE2PY_CMD_RESULT
            )
        endif()

        list(APPEND LIB_FILES ${SLICE2CPP_OUTPUT_FILE}.cpp)
        list(APPEND LIB_HEADER_FILES ${SLICE2CPP_OUTPUT_FILE}.h)
        list(APPEND LIB_HEADER_FILES ${SLICE2CPP_INPUT_FILE})
    endforeach()

    # TODO: Remove? This is used in ArmarXCore, RobotAPI and armarx_ros_core.
    if(SLICE_FILES_ADDITIONAL_HEADERS)
        list(APPEND LIB_HEADER_FILES ${SLICE_FILES_ADDITIONAL_HEADERS})
    endif()
    if(SLICE_FILES_ADDITIONAL_SOURCES)
        list(APPEND LIB_FILES ${SLICE_FILES_ADDITIONAL_SOURCES})
    endif()

    # BUILD_INTERFACE_LIBRARY evaluates to FALSE if length is 0
    list(LENGTH LIB_FILES BUILD_INTERFACE_LIBRARY)

    # Only build the library if all files have been generated.
    if(${BUILD_INTERFACE_LIBRARY})
        list(APPEND DEPENDEND_INTERFACE_LIBRARIES ${CMAKE_THREAD_LIBS_INIT})

        armarx_add_library(${TARGET}
            SOURCES ${LIB_FILES}
            HEADERS ${LIB_HEADER_FILES}
            DEPENDENCIES ${AX_DEPENDENCIES} Ice::Ice
            DEPENDENCIES_LEGACY ${AX_DEPENDENCIES_LEGACY}
            SHARED
        )

        # for dependencies, the source directory of this project should also be available
        list(APPEND SLICE2CPP_INCLUDE_DIRECTORIES "${PROJECT_SOURCECODE_DIR}")
        list(APPEND SLICE2CPP_INCLUDE_DIRECTORIES "${PROJECT_BINARY_DIR}/source")

        # Set include directories for slice2cpp as property to target.
        set_target_properties(${TARGET}
            PROPERTIES
                SLICE2CPP_INCLUDE_DIRECTORIES "${SLICE2CPP_INCLUDE_DIRECTORIES}"
        )

        # ensure that the property SLICE2CPP_INCLUDE_DIRECTORIES is exported correctly to XYTargets.cmake
        set_property(
            TARGET ${TARGET}
            APPEND
            PROPERTY EXPORT_PROPERTIES "SLICE2CPP_INCLUDE_DIRECTORIES"
        )

        target_include_directories(${TARGET} SYSTEM INTERFACE "$<BUILD_INTERFACE:${CMAKE_BINARY_DIR}/source>")

        if(COMPILER_SUPPORTS_suggest_override)
            target_compile_options(${TARGET} PRIVATE -Wno-suggest-override)
        endif()
    endif()

    set(SLICE2HTML_INPUT_FILES "")
    foreach(CURRENT_SLICE_FILE ${SLICE_FILES})
        list(APPEND SLICE2HTML_INPUT_FILES "${SLICE_INPUT_DIR}/${CURRENT_SLICE_FILE}")
    endforeach()

    _armarx_generate_ice_library_export(
        EXPORT_PATH ${CMAKE_CURRENT_BINARY_DIR}
        INCLUDE_DIRECTORIES ${SLICE2CPP_INCLUDE_DIRECTORIES}
    )
endfunction()


function(_armarx_generate_ice_library_export)
    # Parse arguments.
    set(multi_param INCLUDE_DIRECTORIES)
    set(single_param EXPORT_PATH)
    set(flag_param)
    cmake_parse_arguments(PARSE_ARGV 0 AX "${flag_param}" "${single_param}" "${multi_param}")
    if(DEFINED AX_UNPARSED_ARGUMENTS)
        message(FATAL_ERROR "${TARGET}: Unknown arguments `${AX_UNPARSED_ARGUMENTS}`.")
    endif()

    set(FILE "${AX_EXPORT_PATH}/ice_include_directories.txt")

    file(WRITE "${FILE}" "")
    foreach(INCLUDE_DIRECTORY ${AX_INCLUDE_DIRECTORIES})
        file(APPEND "${FILE}" "${INCLUDE_DIRECTORY}\n")
    endforeach()
endfunction()
