/*
* This file is part of ArmarX.
*
* Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarXCore::StatechartExamples::RemoteAccessableStateExample::
* @author     Mirko Waechter ( mirko.waechter at kit dot edu)
* @date       20122012
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once


#include <ArmarXCore/core/system/ImportExportComponent.h>
#include <ArmarXCore/statechart/Statechart.h>

namespace armarx
{
    class ARMARXCOMPONENT_IMPORT_EXPORT RemoteAccessableState :
        virtual public RemoteStateOfferer<StatechartContext>
    {
    public:
        // inherited from Component
        std::string
        getStateOffererName() const override
        {
            return "RemoteAccessable";
        }

        // inherited from RemoteStateOfferer
        void onInitRemoteStateOfferer() override;
    };

    struct Stateadd_x_to_y : StateTemplate<Stateadd_x_to_y>
    {
        void
        defineParameters() override
        {
            addToInput("x", VariantType::Float, false);
            addToInput("y", VariantType::Float, false);

            addToOutput("result", VariantType::Float, false);
        }

        void
        defineSubstates() override
        {
            setInitState(addState<State>("DummySubstate1"));
            addState<State>("DummySubstate2");
        }

        void
        onEnter() override
        {
            float result = getInput<float>("x") + getInput<float>("y");
            setOutput("result", result);
            ARMARX_INFO << "result: " << result << std::endl;
            sendEvent<Success>();
        }
    };


} // namespace armarx
