/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarXCore::ArmarXObjects::ArmarXFileLogger
 * @author     Mirko Waechter ( mirko dot waechter at kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


#include <fstream>
#include <mutex>

#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/interface/core/Log.h>

namespace armarx
{
    /**
     * @class ArmarXFileLoggerPropertyDefinitions
     * @brief
     */
    class ArmarXFileLoggerPropertyDefinitions : public armarx::ComponentPropertyDefinitions
    {
    public:
        ArmarXFileLoggerPropertyDefinitions(std::string prefix);
    };

    /**
     * @defgroup Component-ArmarXFileLogger ArmarXFileLogger
     * @ingroup ArmarXCore-Components
     * A description of the component ArmarXFileLogger.
     *
     * @class ArmarXFileLogger
     * @ingroup Component-ArmarXFileLogger
     * @brief Brief description of class ArmarXFileLogger.
     *
     * Detailed description of class ArmarXFileLogger.
     */
    class ArmarXFileLogger : virtual public armarx::Component, public Log
    {
    public:
        std::string getDefaultName() const override;

        // Log interface
        void writeLog(const LogMessage&, const Ice::Current&) override;


    protected:
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;
        void onInitComponent() override;
        void onConnectComponent() override;
        void onDisconnectComponent() override;
        void onExitComponent() override;

        std::vector<std::string> getDirListOrderedByDate(const std::string& dir) const;
        std::string getNextLogFileNameAndDeleteOldFiles(const std::string& dir,
                                                        int maxLogFileCount,
                                                        const std::string& fileBasename,
                                                        const std::string& fileNameEnding) const;

        std::ofstream fileStream;
        std::string applicationsLogFileBasePath;
        std::map<std::string, std::shared_ptr<std::ofstream>> applicationFileStreams;
        std::mutex mutex;
    };
} // namespace armarx
