#pragma once

#include <memory>
#include <optional>
#include <ostream>

#include <Eigen/Core>

#include <ArmarXCore/util/CPPUtility/trace.h>

namespace armarx::core::eigen
{
    // optional
    template <class EigenT, int rows, int cols>
    std::optional<Eigen::Matrix<EigenT, rows, cols, Eigen::ColMajor>>&
    assign(std::optional<Eigen::Matrix<EigenT, rows, cols, Eigen::ColMajor>>& lh,
           const std::optional<Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>>& rh)
    {
        ARMARX_TRACE;
        if (rh.has_value())
        {
            lh.emplace(rh.value());
        }
        else
        {
            lh = std::nullopt;
        }
        return lh;
    }

    template <class EigenT, int rows, int cols>
    std::optional<Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>>&
    assign(std::optional<Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>>& lh,
           const std::optional<Eigen::Matrix<EigenT, rows, cols, Eigen::ColMajor>>& rh)
    {
        ARMARX_TRACE;
        if (rh.has_value())
        {
            lh.emplace(rh.value());
        }
        else
        {
            lh = std::nullopt;
        }
        return lh;
    }

    // shared_ptr
    template <class EigenT, int rows, int cols>
    std::shared_ptr<Eigen::Matrix<EigenT, rows, cols, Eigen::ColMajor>>&
    assign(std::shared_ptr<Eigen::Matrix<EigenT, rows, cols, Eigen::ColMajor>>& lh,
           const std::shared_ptr<Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>>& rh)
    {
        ARMARX_TRACE;
        if (rh)
        {
            lh = std::make_shared<Eigen::Matrix<EigenT, rows, cols, Eigen::ColMajor>>(*rh);
        }
        else
        {
            lh = nullptr;
        }
        return lh;
    }

    template <class EigenT, int rows, int cols>
    std::shared_ptr<Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>>&
    assign(std::shared_ptr<Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>>& lh,
           const std::shared_ptr<Eigen::Matrix<EigenT, rows, cols, Eigen::ColMajor>>& rh)
    {
        ARMARX_TRACE;
        if (rh)
        {
            lh = std::make_shared<Eigen::Matrix<EigenT, rows, cols, Eigen::RowMajor>>(*rh);
        }
        else
        {
            lh = nullptr;
        }
        return lh;
    }
} // namespace armarx::core::eigen

namespace armarx
{
    using armarx::core::eigen::assign;
}
