/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarXCore::core::Test
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#define BOOST_TEST_MODULE ArmarX::Core::BasicVectorTypes

#define ARMARX_BOOST_TEST

#include <ArmarXCore/Test.h>

#include <iostream>
#include <thread>

#include <ArmarXCore/interface/core/BasicVectorTypesHelpers.h>

#define test()

BOOST_AUTO_TEST_CASE(ToBasicVectorTypeTest)
{
    //this test was generated by applying cpp on the code below (this was done to reduce build time)
#if 0
#define type_test(BType, Type, R, C)                                                               \
    static_assert(                                                                                 \
        std::is_same<BType,                                                                        \
                     decltype(armarx::ToBasicVectorType(::Eigen::Matrix<Type, R, C>()))>::value,   \
        "")

#define make_type_test_1_6(Pre, Suff, Type, i)                                                     \
    type_test(Pre##1##Suff, Type, 1, i);                                                           \
    type_test(Pre##2##Suff, Type, 2, i);                                                           \
    type_test(Pre##3##Suff, Type, 3, i);                                                           \
    type_test(Pre##4##Suff, Type, 4, i);                                                           \
    type_test(Pre##5##Suff, Type, 5, i);                                                           \
    type_test(Pre##6##Suff, Type, 6, i)

#define make_vars(pre, suff, npre, nsuff, val)                                                     \
    const pre##1##suff npre##1##nsuff{val};                                                        \
    const pre##2##suff npre##2##nsuff{val, val};                                                   \
    const pre##3##suff npre##3##nsuff{val, val, val};                                              \
    const pre##4##suff npre##4##nsuff{val, val, val, val};                                         \
    const pre##5##suff npre##5##nsuff{val, val, val, val, val};                                    \
    const pre##6##suff npre##6##nsuff{val, val, val, val, val, val}

#define make_conversion_checks_e2b(Type, vpre, vsuff, i)                                           \
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<Type, 1, i>::Ones() * 3)),        \
                      vpre##1##vsuff);                                                             \
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<Type, 2, i>::Ones() * 3)),        \
                      vpre##2##vsuff);                                                             \
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<Type, 3, i>::Ones() * 3)),        \
                      vpre##3##vsuff);                                                             \
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<Type, 4, i>::Ones() * 3)),        \
                      vpre##4##vsuff);                                                             \
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<Type, 5, i>::Ones() * 3)),        \
                      vpre##5##vsuff);                                                             \
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<Type, 6, i>::Ones() * 3)),        \
                      vpre##6##vsuff)

#define make_conversion_checks_b2e(Type, vpre, vsuff, i)                                           \
    BOOST_CHECK_EQUAL((armarx::ToEigen(vpre##1##vsuff)),                                           \
                      (::Eigen::Matrix<Type, 1, i>::Ones() * 3));                                  \
    BOOST_CHECK_EQUAL((armarx::ToEigen(vpre##2##vsuff)),                                           \
                      (::Eigen::Matrix<Type, 2, i>::Ones() * 3));                                  \
    BOOST_CHECK_EQUAL((armarx::ToEigen(vpre##3##vsuff)),                                           \
                      (::Eigen::Matrix<Type, 3, i>::Ones() * 3));                                  \
    BOOST_CHECK_EQUAL((armarx::ToEigen(vpre##4##vsuff)),                                           \
                      (::Eigen::Matrix<Type, 4, i>::Ones() * 3));                                  \
    BOOST_CHECK_EQUAL((armarx::ToEigen(vpre##5##vsuff)),                                           \
                      (::Eigen::Matrix<Type, 5, i>::Ones() * 3));                                  \
    BOOST_CHECK_EQUAL((armarx::ToEigen(vpre##6##vsuff)), (::Eigen::Matrix<Type, 6, i>::Ones() * 3))

#define make_test_for_type_suffix_combination(Type, Suff)                                          \
    make_type_test_1_6(armarx::Vector, Suff, Type, 1);                                             \
    make_type_test_1_6(armarx::Matrix_, _2_##Suff, Type, 2);                                       \
    make_type_test_1_6(armarx::Matrix_, _3_##Suff, Type, 3);                                       \
    make_type_test_1_6(armarx::Matrix_, _4_##Suff, Type, 4);                                       \
    make_type_test_1_6(armarx::Matrix_, _5_##Suff, Type, 5);                                       \
    make_type_test_1_6(armarx::Matrix_, _6_##Suff, Type, 6);                                       \
                                                                                                   \
    make_vars(armarx::Vector, Suff, v, Suff, 3);                                                   \
    make_vars(armarx::Matrix_, _1_##Suff, m, 1##Suff, v1##Suff);                                   \
    make_vars(armarx::Matrix_, _2_##Suff, m, 2##Suff, v2##Suff);                                   \
    make_vars(armarx::Matrix_, _3_##Suff, m, 3##Suff, v3##Suff);                                   \
    make_vars(armarx::Matrix_, _4_##Suff, m, 4##Suff, v4##Suff);                                   \
    make_vars(armarx::Matrix_, _5_##Suff, m, 5##Suff, v5##Suff);                                   \
    make_vars(armarx::Matrix_, _6_##Suff, m, 6##Suff, v6##Suff);                                   \
                                                                                                   \
    make_conversion_checks_b2e(Type, v, Suff, 1);                                                  \
    make_conversion_checks_b2e(Type, m, 1##Suff, 1);                                               \
    make_conversion_checks_b2e(Type, m, 2##Suff, 2);                                               \
    make_conversion_checks_b2e(Type, m, 3##Suff, 3);                                               \
    make_conversion_checks_b2e(Type, m, 4##Suff, 4);                                               \
    make_conversion_checks_b2e(Type, m, 5##Suff, 5);                                               \
    make_conversion_checks_b2e(Type, m, 6##Suff, 6);                                               \
                                                                                                   \
    make_conversion_checks_e2b(Type, v, Suff, 1);                                                  \
    make_conversion_checks_e2b(Type, m, 2##Suff, 2);                                               \
    make_conversion_checks_e2b(Type, m, 3##Suff, 3);                                               \
    make_conversion_checks_e2b(Type, m, 4##Suff, 4);                                               \
    make_conversion_checks_e2b(Type, m, 5##Suff, 5);                                               \
    make_conversion_checks_e2b(Type, m, 6##Suff, 6)

    make_test_for_type_suffix_combination(::Ice::Byte, byte);
    make_test_for_type_suffix_combination(::Ice::Short, s);
    make_test_for_type_suffix_combination(::Ice::Int, i);
    make_test_for_type_suffix_combination(::Ice::Long, l);
    make_test_for_type_suffix_combination(::Ice::Float, f);
    make_test_for_type_suffix_combination(::Ice::Double, d);
#endif
    static_assert(std::is_same<armarx::Vector1byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 1, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector2byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 2, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector3byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 3, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector4byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 4, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector5byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 5, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector6byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 6, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_2_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 1, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_2_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 2, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_2_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 3, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_2_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 4, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_2_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 5, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_2_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 6, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_3_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 1, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_3_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 2, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_3_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 3, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_3_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 4, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_3_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 5, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_3_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 6, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_4_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 1, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_4_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 2, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_4_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 3, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_4_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 4, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_4_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 5, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_4_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 6, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_5_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 1, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_5_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 2, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_5_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 3, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_5_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 4, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_5_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 5, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_5_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 6, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_6_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 1, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_6_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 2, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_6_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 3, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_6_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 4, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_6_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 5, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_6_byte,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Byte, 6, 6>()))>::value,
                  "");
    const armarx::Vector1byte v1byte{3};
    const armarx::Vector2byte v2byte{3, 3};
    const armarx::Vector3byte v3byte{3, 3, 3};
    const armarx::Vector4byte v4byte{3, 3, 3, 3};
    const armarx::Vector5byte v5byte{3, 3, 3, 3, 3};
    const armarx::Vector6byte v6byte{3, 3, 3, 3, 3, 3};
    const armarx::Matrix_1_1_byte m11byte{v1byte};
    const armarx::Matrix_2_1_byte m21byte{v1byte, v1byte};
    const armarx::Matrix_3_1_byte m31byte{v1byte, v1byte, v1byte};
    const armarx::Matrix_4_1_byte m41byte{v1byte, v1byte, v1byte, v1byte};
    const armarx::Matrix_5_1_byte m51byte{v1byte, v1byte, v1byte, v1byte, v1byte};
    const armarx::Matrix_6_1_byte m61byte{v1byte, v1byte, v1byte, v1byte, v1byte, v1byte};
    const armarx::Matrix_1_2_byte m12byte{v2byte};
    const armarx::Matrix_2_2_byte m22byte{v2byte, v2byte};
    const armarx::Matrix_3_2_byte m32byte{v2byte, v2byte, v2byte};
    const armarx::Matrix_4_2_byte m42byte{v2byte, v2byte, v2byte, v2byte};
    const armarx::Matrix_5_2_byte m52byte{v2byte, v2byte, v2byte, v2byte, v2byte};
    const armarx::Matrix_6_2_byte m62byte{v2byte, v2byte, v2byte, v2byte, v2byte, v2byte};
    const armarx::Matrix_1_3_byte m13byte{v3byte};
    const armarx::Matrix_2_3_byte m23byte{v3byte, v3byte};
    const armarx::Matrix_3_3_byte m33byte{v3byte, v3byte, v3byte};
    const armarx::Matrix_4_3_byte m43byte{v3byte, v3byte, v3byte, v3byte};
    const armarx::Matrix_5_3_byte m53byte{v3byte, v3byte, v3byte, v3byte, v3byte};
    const armarx::Matrix_6_3_byte m63byte{v3byte, v3byte, v3byte, v3byte, v3byte, v3byte};
    const armarx::Matrix_1_4_byte m14byte{v4byte};
    const armarx::Matrix_2_4_byte m24byte{v4byte, v4byte};
    const armarx::Matrix_3_4_byte m34byte{v4byte, v4byte, v4byte};
    const armarx::Matrix_4_4_byte m44byte{v4byte, v4byte, v4byte, v4byte};
    const armarx::Matrix_5_4_byte m54byte{v4byte, v4byte, v4byte, v4byte, v4byte};
    const armarx::Matrix_6_4_byte m64byte{v4byte, v4byte, v4byte, v4byte, v4byte, v4byte};
    const armarx::Matrix_1_5_byte m15byte{v5byte};
    const armarx::Matrix_2_5_byte m25byte{v5byte, v5byte};
    const armarx::Matrix_3_5_byte m35byte{v5byte, v5byte, v5byte};
    const armarx::Matrix_4_5_byte m45byte{v5byte, v5byte, v5byte, v5byte};
    const armarx::Matrix_5_5_byte m55byte{v5byte, v5byte, v5byte, v5byte, v5byte};
    const armarx::Matrix_6_5_byte m65byte{v5byte, v5byte, v5byte, v5byte, v5byte, v5byte};
    const armarx::Matrix_1_6_byte m16byte{v6byte};
    const armarx::Matrix_2_6_byte m26byte{v6byte, v6byte};
    const armarx::Matrix_3_6_byte m36byte{v6byte, v6byte, v6byte};
    const armarx::Matrix_4_6_byte m46byte{v6byte, v6byte, v6byte, v6byte};
    const armarx::Matrix_5_6_byte m56byte{v6byte, v6byte, v6byte, v6byte, v6byte};
    const armarx::Matrix_6_6_byte m66byte{v6byte, v6byte, v6byte, v6byte, v6byte, v6byte};
    BOOST_CHECK_EQUAL((armarx::ToEigen(v1byte)), (::Eigen::Matrix<::Ice::Byte, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v2byte)), (::Eigen::Matrix<::Ice::Byte, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v3byte)), (::Eigen::Matrix<::Ice::Byte, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v4byte)), (::Eigen::Matrix<::Ice::Byte, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v5byte)), (::Eigen::Matrix<::Ice::Byte, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v6byte)), (::Eigen::Matrix<::Ice::Byte, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m11byte)), (::Eigen::Matrix<::Ice::Byte, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m21byte)), (::Eigen::Matrix<::Ice::Byte, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m31byte)), (::Eigen::Matrix<::Ice::Byte, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m41byte)), (::Eigen::Matrix<::Ice::Byte, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m51byte)), (::Eigen::Matrix<::Ice::Byte, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m61byte)), (::Eigen::Matrix<::Ice::Byte, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m12byte)), (::Eigen::Matrix<::Ice::Byte, 1, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m22byte)), (::Eigen::Matrix<::Ice::Byte, 2, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m32byte)), (::Eigen::Matrix<::Ice::Byte, 3, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m42byte)), (::Eigen::Matrix<::Ice::Byte, 4, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m52byte)), (::Eigen::Matrix<::Ice::Byte, 5, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m62byte)), (::Eigen::Matrix<::Ice::Byte, 6, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m13byte)), (::Eigen::Matrix<::Ice::Byte, 1, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m23byte)), (::Eigen::Matrix<::Ice::Byte, 2, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m33byte)), (::Eigen::Matrix<::Ice::Byte, 3, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m43byte)), (::Eigen::Matrix<::Ice::Byte, 4, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m53byte)), (::Eigen::Matrix<::Ice::Byte, 5, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m63byte)), (::Eigen::Matrix<::Ice::Byte, 6, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m14byte)), (::Eigen::Matrix<::Ice::Byte, 1, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m24byte)), (::Eigen::Matrix<::Ice::Byte, 2, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m34byte)), (::Eigen::Matrix<::Ice::Byte, 3, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m44byte)), (::Eigen::Matrix<::Ice::Byte, 4, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m54byte)), (::Eigen::Matrix<::Ice::Byte, 5, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m64byte)), (::Eigen::Matrix<::Ice::Byte, 6, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m15byte)), (::Eigen::Matrix<::Ice::Byte, 1, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m25byte)), (::Eigen::Matrix<::Ice::Byte, 2, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m35byte)), (::Eigen::Matrix<::Ice::Byte, 3, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m45byte)), (::Eigen::Matrix<::Ice::Byte, 4, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m55byte)), (::Eigen::Matrix<::Ice::Byte, 5, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m65byte)), (::Eigen::Matrix<::Ice::Byte, 6, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m16byte)), (::Eigen::Matrix<::Ice::Byte, 1, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m26byte)), (::Eigen::Matrix<::Ice::Byte, 2, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m36byte)), (::Eigen::Matrix<::Ice::Byte, 3, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m46byte)), (::Eigen::Matrix<::Ice::Byte, 4, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m56byte)), (::Eigen::Matrix<::Ice::Byte, 5, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m66byte)), (::Eigen::Matrix<::Ice::Byte, 6, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 1, 1>::Ones() * 3)),
                      v1byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 2, 1>::Ones() * 3)),
                      v2byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 3, 1>::Ones() * 3)),
                      v3byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 4, 1>::Ones() * 3)),
                      v4byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 5, 1>::Ones() * 3)),
                      v5byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 6, 1>::Ones() * 3)),
                      v6byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 1, 2>::Ones() * 3)),
                      m12byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 2, 2>::Ones() * 3)),
                      m22byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 3, 2>::Ones() * 3)),
                      m32byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 4, 2>::Ones() * 3)),
                      m42byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 5, 2>::Ones() * 3)),
                      m52byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 6, 2>::Ones() * 3)),
                      m62byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 1, 3>::Ones() * 3)),
                      m13byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 2, 3>::Ones() * 3)),
                      m23byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 3, 3>::Ones() * 3)),
                      m33byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 4, 3>::Ones() * 3)),
                      m43byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 5, 3>::Ones() * 3)),
                      m53byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 6, 3>::Ones() * 3)),
                      m63byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 1, 4>::Ones() * 3)),
                      m14byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 2, 4>::Ones() * 3)),
                      m24byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 3, 4>::Ones() * 3)),
                      m34byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 4, 4>::Ones() * 3)),
                      m44byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 5, 4>::Ones() * 3)),
                      m54byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 6, 4>::Ones() * 3)),
                      m64byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 1, 5>::Ones() * 3)),
                      m15byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 2, 5>::Ones() * 3)),
                      m25byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 3, 5>::Ones() * 3)),
                      m35byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 4, 5>::Ones() * 3)),
                      m45byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 5, 5>::Ones() * 3)),
                      m55byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 6, 5>::Ones() * 3)),
                      m65byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 1, 6>::Ones() * 3)),
                      m16byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 2, 6>::Ones() * 3)),
                      m26byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 3, 6>::Ones() * 3)),
                      m36byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 4, 6>::Ones() * 3)),
                      m46byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 5, 6>::Ones() * 3)),
                      m56byte);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Byte, 6, 6>::Ones() * 3)),
                      m66byte);
    static_assert(std::is_same<armarx::Vector1s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 1, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector2s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 2, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector3s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 3, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector4s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 4, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector5s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 5, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector6s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 6, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_2_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 1, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_2_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 2, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_2_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 3, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_2_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 4, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_2_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 5, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_2_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 6, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_3_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 1, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_3_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 2, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_3_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 3, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_3_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 4, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_3_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 5, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_3_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 6, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_4_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 1, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_4_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 2, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_4_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 3, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_4_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 4, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_4_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 5, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_4_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 6, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_5_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 1, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_5_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 2, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_5_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 3, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_5_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 4, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_5_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 5, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_5_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 6, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_6_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 1, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_6_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 2, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_6_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 3, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_6_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 4, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_6_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 5, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_6_s,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Short, 6, 6>()))>::value,
                  "");
    const armarx::Vector1s v1s{3};
    const armarx::Vector2s v2s{3, 3};
    const armarx::Vector3s v3s{3, 3, 3};
    const armarx::Vector4s v4s{3, 3, 3, 3};
    const armarx::Vector5s v5s{3, 3, 3, 3, 3};
    const armarx::Vector6s v6s{3, 3, 3, 3, 3, 3};
    const armarx::Matrix_1_1_s m11s{v1s};
    const armarx::Matrix_2_1_s m21s{v1s, v1s};
    const armarx::Matrix_3_1_s m31s{v1s, v1s, v1s};
    const armarx::Matrix_4_1_s m41s{v1s, v1s, v1s, v1s};
    const armarx::Matrix_5_1_s m51s{v1s, v1s, v1s, v1s, v1s};
    const armarx::Matrix_6_1_s m61s{v1s, v1s, v1s, v1s, v1s, v1s};
    const armarx::Matrix_1_2_s m12s{v2s};
    const armarx::Matrix_2_2_s m22s{v2s, v2s};
    const armarx::Matrix_3_2_s m32s{v2s, v2s, v2s};
    const armarx::Matrix_4_2_s m42s{v2s, v2s, v2s, v2s};
    const armarx::Matrix_5_2_s m52s{v2s, v2s, v2s, v2s, v2s};
    const armarx::Matrix_6_2_s m62s{v2s, v2s, v2s, v2s, v2s, v2s};
    const armarx::Matrix_1_3_s m13s{v3s};
    const armarx::Matrix_2_3_s m23s{v3s, v3s};
    const armarx::Matrix_3_3_s m33s{v3s, v3s, v3s};
    const armarx::Matrix_4_3_s m43s{v3s, v3s, v3s, v3s};
    const armarx::Matrix_5_3_s m53s{v3s, v3s, v3s, v3s, v3s};
    const armarx::Matrix_6_3_s m63s{v3s, v3s, v3s, v3s, v3s, v3s};
    const armarx::Matrix_1_4_s m14s{v4s};
    const armarx::Matrix_2_4_s m24s{v4s, v4s};
    const armarx::Matrix_3_4_s m34s{v4s, v4s, v4s};
    const armarx::Matrix_4_4_s m44s{v4s, v4s, v4s, v4s};
    const armarx::Matrix_5_4_s m54s{v4s, v4s, v4s, v4s, v4s};
    const armarx::Matrix_6_4_s m64s{v4s, v4s, v4s, v4s, v4s, v4s};
    const armarx::Matrix_1_5_s m15s{v5s};
    const armarx::Matrix_2_5_s m25s{v5s, v5s};
    const armarx::Matrix_3_5_s m35s{v5s, v5s, v5s};
    const armarx::Matrix_4_5_s m45s{v5s, v5s, v5s, v5s};
    const armarx::Matrix_5_5_s m55s{v5s, v5s, v5s, v5s, v5s};
    const armarx::Matrix_6_5_s m65s{v5s, v5s, v5s, v5s, v5s, v5s};
    const armarx::Matrix_1_6_s m16s{v6s};
    const armarx::Matrix_2_6_s m26s{v6s, v6s};
    const armarx::Matrix_3_6_s m36s{v6s, v6s, v6s};
    const armarx::Matrix_4_6_s m46s{v6s, v6s, v6s, v6s};
    const armarx::Matrix_5_6_s m56s{v6s, v6s, v6s, v6s, v6s};
    const armarx::Matrix_6_6_s m66s{v6s, v6s, v6s, v6s, v6s, v6s};
    BOOST_CHECK_EQUAL((armarx::ToEigen(v1s)), (::Eigen::Matrix<::Ice::Short, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v2s)), (::Eigen::Matrix<::Ice::Short, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v3s)), (::Eigen::Matrix<::Ice::Short, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v4s)), (::Eigen::Matrix<::Ice::Short, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v5s)), (::Eigen::Matrix<::Ice::Short, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v6s)), (::Eigen::Matrix<::Ice::Short, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m11s)), (::Eigen::Matrix<::Ice::Short, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m21s)), (::Eigen::Matrix<::Ice::Short, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m31s)), (::Eigen::Matrix<::Ice::Short, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m41s)), (::Eigen::Matrix<::Ice::Short, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m51s)), (::Eigen::Matrix<::Ice::Short, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m61s)), (::Eigen::Matrix<::Ice::Short, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m12s)), (::Eigen::Matrix<::Ice::Short, 1, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m22s)), (::Eigen::Matrix<::Ice::Short, 2, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m32s)), (::Eigen::Matrix<::Ice::Short, 3, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m42s)), (::Eigen::Matrix<::Ice::Short, 4, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m52s)), (::Eigen::Matrix<::Ice::Short, 5, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m62s)), (::Eigen::Matrix<::Ice::Short, 6, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m13s)), (::Eigen::Matrix<::Ice::Short, 1, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m23s)), (::Eigen::Matrix<::Ice::Short, 2, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m33s)), (::Eigen::Matrix<::Ice::Short, 3, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m43s)), (::Eigen::Matrix<::Ice::Short, 4, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m53s)), (::Eigen::Matrix<::Ice::Short, 5, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m63s)), (::Eigen::Matrix<::Ice::Short, 6, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m14s)), (::Eigen::Matrix<::Ice::Short, 1, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m24s)), (::Eigen::Matrix<::Ice::Short, 2, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m34s)), (::Eigen::Matrix<::Ice::Short, 3, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m44s)), (::Eigen::Matrix<::Ice::Short, 4, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m54s)), (::Eigen::Matrix<::Ice::Short, 5, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m64s)), (::Eigen::Matrix<::Ice::Short, 6, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m15s)), (::Eigen::Matrix<::Ice::Short, 1, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m25s)), (::Eigen::Matrix<::Ice::Short, 2, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m35s)), (::Eigen::Matrix<::Ice::Short, 3, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m45s)), (::Eigen::Matrix<::Ice::Short, 4, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m55s)), (::Eigen::Matrix<::Ice::Short, 5, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m65s)), (::Eigen::Matrix<::Ice::Short, 6, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m16s)), (::Eigen::Matrix<::Ice::Short, 1, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m26s)), (::Eigen::Matrix<::Ice::Short, 2, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m36s)), (::Eigen::Matrix<::Ice::Short, 3, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m46s)), (::Eigen::Matrix<::Ice::Short, 4, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m56s)), (::Eigen::Matrix<::Ice::Short, 5, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m66s)), (::Eigen::Matrix<::Ice::Short, 6, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 1, 1>::Ones() * 3)),
                      v1s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 2, 1>::Ones() * 3)),
                      v2s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 3, 1>::Ones() * 3)),
                      v3s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 4, 1>::Ones() * 3)),
                      v4s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 5, 1>::Ones() * 3)),
                      v5s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 6, 1>::Ones() * 3)),
                      v6s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 1, 2>::Ones() * 3)),
                      m12s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 2, 2>::Ones() * 3)),
                      m22s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 3, 2>::Ones() * 3)),
                      m32s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 4, 2>::Ones() * 3)),
                      m42s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 5, 2>::Ones() * 3)),
                      m52s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 6, 2>::Ones() * 3)),
                      m62s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 1, 3>::Ones() * 3)),
                      m13s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 2, 3>::Ones() * 3)),
                      m23s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 3, 3>::Ones() * 3)),
                      m33s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 4, 3>::Ones() * 3)),
                      m43s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 5, 3>::Ones() * 3)),
                      m53s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 6, 3>::Ones() * 3)),
                      m63s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 1, 4>::Ones() * 3)),
                      m14s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 2, 4>::Ones() * 3)),
                      m24s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 3, 4>::Ones() * 3)),
                      m34s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 4, 4>::Ones() * 3)),
                      m44s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 5, 4>::Ones() * 3)),
                      m54s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 6, 4>::Ones() * 3)),
                      m64s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 1, 5>::Ones() * 3)),
                      m15s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 2, 5>::Ones() * 3)),
                      m25s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 3, 5>::Ones() * 3)),
                      m35s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 4, 5>::Ones() * 3)),
                      m45s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 5, 5>::Ones() * 3)),
                      m55s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 6, 5>::Ones() * 3)),
                      m65s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 1, 6>::Ones() * 3)),
                      m16s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 2, 6>::Ones() * 3)),
                      m26s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 3, 6>::Ones() * 3)),
                      m36s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 4, 6>::Ones() * 3)),
                      m46s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 5, 6>::Ones() * 3)),
                      m56s);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Short, 6, 6>::Ones() * 3)),
                      m66s);
    static_assert(std::is_same<armarx::Vector1i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 1, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector2i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 2, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector3i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 3, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector4i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 4, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector5i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 5, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector6i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 6, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_2_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 1, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_2_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 2, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_2_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 3, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_2_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 4, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_2_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 5, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_2_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 6, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_3_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 1, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_3_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 2, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_3_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 3, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_3_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 4, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_3_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 5, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_3_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 6, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_4_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 1, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_4_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 2, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_4_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 3, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_4_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 4, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_4_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 5, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_4_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 6, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_5_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 1, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_5_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 2, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_5_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 3, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_5_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 4, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_5_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 5, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_5_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 6, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_6_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 1, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_6_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 2, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_6_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 3, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_6_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 4, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_6_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 5, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_6_i,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Int, 6, 6>()))>::value,
                  "");
    const armarx::Vector1i v1i{3};
    const armarx::Vector2i v2i{3, 3};
    const armarx::Vector3i v3i{3, 3, 3};
    const armarx::Vector4i v4i{3, 3, 3, 3};
    const armarx::Vector5i v5i{3, 3, 3, 3, 3};
    const armarx::Vector6i v6i{3, 3, 3, 3, 3, 3};
    const armarx::Matrix_1_1_i m11i{v1i};
    const armarx::Matrix_2_1_i m21i{v1i, v1i};
    const armarx::Matrix_3_1_i m31i{v1i, v1i, v1i};
    const armarx::Matrix_4_1_i m41i{v1i, v1i, v1i, v1i};
    const armarx::Matrix_5_1_i m51i{v1i, v1i, v1i, v1i, v1i};
    const armarx::Matrix_6_1_i m61i{v1i, v1i, v1i, v1i, v1i, v1i};
    const armarx::Matrix_1_2_i m12i{v2i};
    const armarx::Matrix_2_2_i m22i{v2i, v2i};
    const armarx::Matrix_3_2_i m32i{v2i, v2i, v2i};
    const armarx::Matrix_4_2_i m42i{v2i, v2i, v2i, v2i};
    const armarx::Matrix_5_2_i m52i{v2i, v2i, v2i, v2i, v2i};
    const armarx::Matrix_6_2_i m62i{v2i, v2i, v2i, v2i, v2i, v2i};
    const armarx::Matrix_1_3_i m13i{v3i};
    const armarx::Matrix_2_3_i m23i{v3i, v3i};
    const armarx::Matrix_3_3_i m33i{v3i, v3i, v3i};
    const armarx::Matrix_4_3_i m43i{v3i, v3i, v3i, v3i};
    const armarx::Matrix_5_3_i m53i{v3i, v3i, v3i, v3i, v3i};
    const armarx::Matrix_6_3_i m63i{v3i, v3i, v3i, v3i, v3i, v3i};
    const armarx::Matrix_1_4_i m14i{v4i};
    const armarx::Matrix_2_4_i m24i{v4i, v4i};
    const armarx::Matrix_3_4_i m34i{v4i, v4i, v4i};
    const armarx::Matrix_4_4_i m44i{v4i, v4i, v4i, v4i};
    const armarx::Matrix_5_4_i m54i{v4i, v4i, v4i, v4i, v4i};
    const armarx::Matrix_6_4_i m64i{v4i, v4i, v4i, v4i, v4i, v4i};
    const armarx::Matrix_1_5_i m15i{v5i};
    const armarx::Matrix_2_5_i m25i{v5i, v5i};
    const armarx::Matrix_3_5_i m35i{v5i, v5i, v5i};
    const armarx::Matrix_4_5_i m45i{v5i, v5i, v5i, v5i};
    const armarx::Matrix_5_5_i m55i{v5i, v5i, v5i, v5i, v5i};
    const armarx::Matrix_6_5_i m65i{v5i, v5i, v5i, v5i, v5i, v5i};
    const armarx::Matrix_1_6_i m16i{v6i};
    const armarx::Matrix_2_6_i m26i{v6i, v6i};
    const armarx::Matrix_3_6_i m36i{v6i, v6i, v6i};
    const armarx::Matrix_4_6_i m46i{v6i, v6i, v6i, v6i};
    const armarx::Matrix_5_6_i m56i{v6i, v6i, v6i, v6i, v6i};
    const armarx::Matrix_6_6_i m66i{v6i, v6i, v6i, v6i, v6i, v6i};
    BOOST_CHECK_EQUAL((armarx::ToEigen(v1i)), (::Eigen::Matrix<::Ice::Int, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v2i)), (::Eigen::Matrix<::Ice::Int, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v3i)), (::Eigen::Matrix<::Ice::Int, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v4i)), (::Eigen::Matrix<::Ice::Int, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v5i)), (::Eigen::Matrix<::Ice::Int, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v6i)), (::Eigen::Matrix<::Ice::Int, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m11i)), (::Eigen::Matrix<::Ice::Int, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m21i)), (::Eigen::Matrix<::Ice::Int, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m31i)), (::Eigen::Matrix<::Ice::Int, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m41i)), (::Eigen::Matrix<::Ice::Int, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m51i)), (::Eigen::Matrix<::Ice::Int, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m61i)), (::Eigen::Matrix<::Ice::Int, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m12i)), (::Eigen::Matrix<::Ice::Int, 1, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m22i)), (::Eigen::Matrix<::Ice::Int, 2, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m32i)), (::Eigen::Matrix<::Ice::Int, 3, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m42i)), (::Eigen::Matrix<::Ice::Int, 4, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m52i)), (::Eigen::Matrix<::Ice::Int, 5, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m62i)), (::Eigen::Matrix<::Ice::Int, 6, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m13i)), (::Eigen::Matrix<::Ice::Int, 1, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m23i)), (::Eigen::Matrix<::Ice::Int, 2, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m33i)), (::Eigen::Matrix<::Ice::Int, 3, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m43i)), (::Eigen::Matrix<::Ice::Int, 4, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m53i)), (::Eigen::Matrix<::Ice::Int, 5, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m63i)), (::Eigen::Matrix<::Ice::Int, 6, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m14i)), (::Eigen::Matrix<::Ice::Int, 1, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m24i)), (::Eigen::Matrix<::Ice::Int, 2, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m34i)), (::Eigen::Matrix<::Ice::Int, 3, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m44i)), (::Eigen::Matrix<::Ice::Int, 4, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m54i)), (::Eigen::Matrix<::Ice::Int, 5, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m64i)), (::Eigen::Matrix<::Ice::Int, 6, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m15i)), (::Eigen::Matrix<::Ice::Int, 1, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m25i)), (::Eigen::Matrix<::Ice::Int, 2, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m35i)), (::Eigen::Matrix<::Ice::Int, 3, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m45i)), (::Eigen::Matrix<::Ice::Int, 4, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m55i)), (::Eigen::Matrix<::Ice::Int, 5, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m65i)), (::Eigen::Matrix<::Ice::Int, 6, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m16i)), (::Eigen::Matrix<::Ice::Int, 1, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m26i)), (::Eigen::Matrix<::Ice::Int, 2, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m36i)), (::Eigen::Matrix<::Ice::Int, 3, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m46i)), (::Eigen::Matrix<::Ice::Int, 4, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m56i)), (::Eigen::Matrix<::Ice::Int, 5, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m66i)), (::Eigen::Matrix<::Ice::Int, 6, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 1, 1>::Ones() * 3)),
                      v1i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 2, 1>::Ones() * 3)),
                      v2i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 3, 1>::Ones() * 3)),
                      v3i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 4, 1>::Ones() * 3)),
                      v4i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 5, 1>::Ones() * 3)),
                      v5i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 6, 1>::Ones() * 3)),
                      v6i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 1, 2>::Ones() * 3)),
                      m12i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 2, 2>::Ones() * 3)),
                      m22i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 3, 2>::Ones() * 3)),
                      m32i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 4, 2>::Ones() * 3)),
                      m42i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 5, 2>::Ones() * 3)),
                      m52i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 6, 2>::Ones() * 3)),
                      m62i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 1, 3>::Ones() * 3)),
                      m13i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 2, 3>::Ones() * 3)),
                      m23i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 3, 3>::Ones() * 3)),
                      m33i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 4, 3>::Ones() * 3)),
                      m43i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 5, 3>::Ones() * 3)),
                      m53i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 6, 3>::Ones() * 3)),
                      m63i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 1, 4>::Ones() * 3)),
                      m14i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 2, 4>::Ones() * 3)),
                      m24i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 3, 4>::Ones() * 3)),
                      m34i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 4, 4>::Ones() * 3)),
                      m44i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 5, 4>::Ones() * 3)),
                      m54i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 6, 4>::Ones() * 3)),
                      m64i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 1, 5>::Ones() * 3)),
                      m15i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 2, 5>::Ones() * 3)),
                      m25i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 3, 5>::Ones() * 3)),
                      m35i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 4, 5>::Ones() * 3)),
                      m45i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 5, 5>::Ones() * 3)),
                      m55i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 6, 5>::Ones() * 3)),
                      m65i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 1, 6>::Ones() * 3)),
                      m16i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 2, 6>::Ones() * 3)),
                      m26i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 3, 6>::Ones() * 3)),
                      m36i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 4, 6>::Ones() * 3)),
                      m46i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 5, 6>::Ones() * 3)),
                      m56i);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Int, 6, 6>::Ones() * 3)),
                      m66i);
    static_assert(std::is_same<armarx::Vector1l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 1, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector2l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 2, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector3l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 3, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector4l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 4, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector5l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 5, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector6l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 6, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_2_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 1, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_2_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 2, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_2_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 3, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_2_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 4, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_2_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 5, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_2_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 6, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_3_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 1, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_3_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 2, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_3_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 3, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_3_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 4, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_3_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 5, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_3_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 6, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_4_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 1, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_4_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 2, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_4_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 3, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_4_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 4, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_4_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 5, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_4_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 6, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_5_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 1, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_5_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 2, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_5_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 3, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_5_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 4, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_5_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 5, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_5_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 6, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_6_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 1, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_6_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 2, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_6_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 3, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_6_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 4, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_6_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 5, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_6_l,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Long, 6, 6>()))>::value,
                  "");
    const armarx::Vector1l v1l{3};
    const armarx::Vector2l v2l{3, 3};
    const armarx::Vector3l v3l{3, 3, 3};
    const armarx::Vector4l v4l{3, 3, 3, 3};
    const armarx::Vector5l v5l{3, 3, 3, 3, 3};
    const armarx::Vector6l v6l{3, 3, 3, 3, 3, 3};
    const armarx::Matrix_1_1_l m11l{v1l};
    const armarx::Matrix_2_1_l m21l{v1l, v1l};
    const armarx::Matrix_3_1_l m31l{v1l, v1l, v1l};
    const armarx::Matrix_4_1_l m41l{v1l, v1l, v1l, v1l};
    const armarx::Matrix_5_1_l m51l{v1l, v1l, v1l, v1l, v1l};
    const armarx::Matrix_6_1_l m61l{v1l, v1l, v1l, v1l, v1l, v1l};
    const armarx::Matrix_1_2_l m12l{v2l};
    const armarx::Matrix_2_2_l m22l{v2l, v2l};
    const armarx::Matrix_3_2_l m32l{v2l, v2l, v2l};
    const armarx::Matrix_4_2_l m42l{v2l, v2l, v2l, v2l};
    const armarx::Matrix_5_2_l m52l{v2l, v2l, v2l, v2l, v2l};
    const armarx::Matrix_6_2_l m62l{v2l, v2l, v2l, v2l, v2l, v2l};
    const armarx::Matrix_1_3_l m13l{v3l};
    const armarx::Matrix_2_3_l m23l{v3l, v3l};
    const armarx::Matrix_3_3_l m33l{v3l, v3l, v3l};
    const armarx::Matrix_4_3_l m43l{v3l, v3l, v3l, v3l};
    const armarx::Matrix_5_3_l m53l{v3l, v3l, v3l, v3l, v3l};
    const armarx::Matrix_6_3_l m63l{v3l, v3l, v3l, v3l, v3l, v3l};
    const armarx::Matrix_1_4_l m14l{v4l};
    const armarx::Matrix_2_4_l m24l{v4l, v4l};
    const armarx::Matrix_3_4_l m34l{v4l, v4l, v4l};
    const armarx::Matrix_4_4_l m44l{v4l, v4l, v4l, v4l};
    const armarx::Matrix_5_4_l m54l{v4l, v4l, v4l, v4l, v4l};
    const armarx::Matrix_6_4_l m64l{v4l, v4l, v4l, v4l, v4l, v4l};
    const armarx::Matrix_1_5_l m15l{v5l};
    const armarx::Matrix_2_5_l m25l{v5l, v5l};
    const armarx::Matrix_3_5_l m35l{v5l, v5l, v5l};
    const armarx::Matrix_4_5_l m45l{v5l, v5l, v5l, v5l};
    const armarx::Matrix_5_5_l m55l{v5l, v5l, v5l, v5l, v5l};
    const armarx::Matrix_6_5_l m65l{v5l, v5l, v5l, v5l, v5l, v5l};
    const armarx::Matrix_1_6_l m16l{v6l};
    const armarx::Matrix_2_6_l m26l{v6l, v6l};
    const armarx::Matrix_3_6_l m36l{v6l, v6l, v6l};
    const armarx::Matrix_4_6_l m46l{v6l, v6l, v6l, v6l};
    const armarx::Matrix_5_6_l m56l{v6l, v6l, v6l, v6l, v6l};
    const armarx::Matrix_6_6_l m66l{v6l, v6l, v6l, v6l, v6l, v6l};
    BOOST_CHECK_EQUAL((armarx::ToEigen(v1l)), (::Eigen::Matrix<::Ice::Long, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v2l)), (::Eigen::Matrix<::Ice::Long, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v3l)), (::Eigen::Matrix<::Ice::Long, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v4l)), (::Eigen::Matrix<::Ice::Long, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v5l)), (::Eigen::Matrix<::Ice::Long, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v6l)), (::Eigen::Matrix<::Ice::Long, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m11l)), (::Eigen::Matrix<::Ice::Long, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m21l)), (::Eigen::Matrix<::Ice::Long, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m31l)), (::Eigen::Matrix<::Ice::Long, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m41l)), (::Eigen::Matrix<::Ice::Long, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m51l)), (::Eigen::Matrix<::Ice::Long, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m61l)), (::Eigen::Matrix<::Ice::Long, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m12l)), (::Eigen::Matrix<::Ice::Long, 1, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m22l)), (::Eigen::Matrix<::Ice::Long, 2, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m32l)), (::Eigen::Matrix<::Ice::Long, 3, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m42l)), (::Eigen::Matrix<::Ice::Long, 4, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m52l)), (::Eigen::Matrix<::Ice::Long, 5, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m62l)), (::Eigen::Matrix<::Ice::Long, 6, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m13l)), (::Eigen::Matrix<::Ice::Long, 1, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m23l)), (::Eigen::Matrix<::Ice::Long, 2, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m33l)), (::Eigen::Matrix<::Ice::Long, 3, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m43l)), (::Eigen::Matrix<::Ice::Long, 4, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m53l)), (::Eigen::Matrix<::Ice::Long, 5, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m63l)), (::Eigen::Matrix<::Ice::Long, 6, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m14l)), (::Eigen::Matrix<::Ice::Long, 1, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m24l)), (::Eigen::Matrix<::Ice::Long, 2, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m34l)), (::Eigen::Matrix<::Ice::Long, 3, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m44l)), (::Eigen::Matrix<::Ice::Long, 4, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m54l)), (::Eigen::Matrix<::Ice::Long, 5, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m64l)), (::Eigen::Matrix<::Ice::Long, 6, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m15l)), (::Eigen::Matrix<::Ice::Long, 1, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m25l)), (::Eigen::Matrix<::Ice::Long, 2, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m35l)), (::Eigen::Matrix<::Ice::Long, 3, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m45l)), (::Eigen::Matrix<::Ice::Long, 4, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m55l)), (::Eigen::Matrix<::Ice::Long, 5, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m65l)), (::Eigen::Matrix<::Ice::Long, 6, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m16l)), (::Eigen::Matrix<::Ice::Long, 1, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m26l)), (::Eigen::Matrix<::Ice::Long, 2, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m36l)), (::Eigen::Matrix<::Ice::Long, 3, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m46l)), (::Eigen::Matrix<::Ice::Long, 4, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m56l)), (::Eigen::Matrix<::Ice::Long, 5, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m66l)), (::Eigen::Matrix<::Ice::Long, 6, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 1, 1>::Ones() * 3)),
                      v1l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 2, 1>::Ones() * 3)),
                      v2l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 3, 1>::Ones() * 3)),
                      v3l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 4, 1>::Ones() * 3)),
                      v4l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 5, 1>::Ones() * 3)),
                      v5l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 6, 1>::Ones() * 3)),
                      v6l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 1, 2>::Ones() * 3)),
                      m12l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 2, 2>::Ones() * 3)),
                      m22l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 3, 2>::Ones() * 3)),
                      m32l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 4, 2>::Ones() * 3)),
                      m42l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 5, 2>::Ones() * 3)),
                      m52l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 6, 2>::Ones() * 3)),
                      m62l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 1, 3>::Ones() * 3)),
                      m13l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 2, 3>::Ones() * 3)),
                      m23l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 3, 3>::Ones() * 3)),
                      m33l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 4, 3>::Ones() * 3)),
                      m43l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 5, 3>::Ones() * 3)),
                      m53l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 6, 3>::Ones() * 3)),
                      m63l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 1, 4>::Ones() * 3)),
                      m14l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 2, 4>::Ones() * 3)),
                      m24l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 3, 4>::Ones() * 3)),
                      m34l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 4, 4>::Ones() * 3)),
                      m44l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 5, 4>::Ones() * 3)),
                      m54l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 6, 4>::Ones() * 3)),
                      m64l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 1, 5>::Ones() * 3)),
                      m15l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 2, 5>::Ones() * 3)),
                      m25l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 3, 5>::Ones() * 3)),
                      m35l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 4, 5>::Ones() * 3)),
                      m45l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 5, 5>::Ones() * 3)),
                      m55l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 6, 5>::Ones() * 3)),
                      m65l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 1, 6>::Ones() * 3)),
                      m16l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 2, 6>::Ones() * 3)),
                      m26l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 3, 6>::Ones() * 3)),
                      m36l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 4, 6>::Ones() * 3)),
                      m46l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 5, 6>::Ones() * 3)),
                      m56l);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Long, 6, 6>::Ones() * 3)),
                      m66l);
    static_assert(std::is_same<armarx::Vector1f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 1, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector2f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 2, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector3f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 3, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector4f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 4, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector5f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 5, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector6f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 6, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_2_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 1, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_2_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 2, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_2_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 3, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_2_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 4, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_2_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 5, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_2_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 6, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_3_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 1, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_3_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 2, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_3_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 3, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_3_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 4, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_3_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 5, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_3_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 6, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_4_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 1, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_4_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 2, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_4_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 3, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_4_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 4, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_4_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 5, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_4_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 6, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_5_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 1, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_5_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 2, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_5_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 3, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_5_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 4, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_5_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 5, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_5_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 6, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_6_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 1, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_6_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 2, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_6_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 3, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_6_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 4, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_6_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 5, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_6_f,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Float, 6, 6>()))>::value,
                  "");
    const armarx::Vector1f v1f{3};
    const armarx::Vector2f v2f{3, 3};
    const armarx::Vector3f v3f{3, 3, 3};
    const armarx::Vector4f v4f{3, 3, 3, 3};
    const armarx::Vector5f v5f{3, 3, 3, 3, 3};
    const armarx::Vector6f v6f{3, 3, 3, 3, 3, 3};
    const armarx::Matrix_1_1_f m11f{v1f};
    const armarx::Matrix_2_1_f m21f{v1f, v1f};
    const armarx::Matrix_3_1_f m31f{v1f, v1f, v1f};
    const armarx::Matrix_4_1_f m41f{v1f, v1f, v1f, v1f};
    const armarx::Matrix_5_1_f m51f{v1f, v1f, v1f, v1f, v1f};
    const armarx::Matrix_6_1_f m61f{v1f, v1f, v1f, v1f, v1f, v1f};
    const armarx::Matrix_1_2_f m12f{v2f};
    const armarx::Matrix_2_2_f m22f{v2f, v2f};
    const armarx::Matrix_3_2_f m32f{v2f, v2f, v2f};
    const armarx::Matrix_4_2_f m42f{v2f, v2f, v2f, v2f};
    const armarx::Matrix_5_2_f m52f{v2f, v2f, v2f, v2f, v2f};
    const armarx::Matrix_6_2_f m62f{v2f, v2f, v2f, v2f, v2f, v2f};
    const armarx::Matrix_1_3_f m13f{v3f};
    const armarx::Matrix_2_3_f m23f{v3f, v3f};
    const armarx::Matrix_3_3_f m33f{v3f, v3f, v3f};
    const armarx::Matrix_4_3_f m43f{v3f, v3f, v3f, v3f};
    const armarx::Matrix_5_3_f m53f{v3f, v3f, v3f, v3f, v3f};
    const armarx::Matrix_6_3_f m63f{v3f, v3f, v3f, v3f, v3f, v3f};
    const armarx::Matrix_1_4_f m14f{v4f};
    const armarx::Matrix_2_4_f m24f{v4f, v4f};
    const armarx::Matrix_3_4_f m34f{v4f, v4f, v4f};
    const armarx::Matrix_4_4_f m44f{v4f, v4f, v4f, v4f};
    const armarx::Matrix_5_4_f m54f{v4f, v4f, v4f, v4f, v4f};
    const armarx::Matrix_6_4_f m64f{v4f, v4f, v4f, v4f, v4f, v4f};
    const armarx::Matrix_1_5_f m15f{v5f};
    const armarx::Matrix_2_5_f m25f{v5f, v5f};
    const armarx::Matrix_3_5_f m35f{v5f, v5f, v5f};
    const armarx::Matrix_4_5_f m45f{v5f, v5f, v5f, v5f};
    const armarx::Matrix_5_5_f m55f{v5f, v5f, v5f, v5f, v5f};
    const armarx::Matrix_6_5_f m65f{v5f, v5f, v5f, v5f, v5f, v5f};
    const armarx::Matrix_1_6_f m16f{v6f};
    const armarx::Matrix_2_6_f m26f{v6f, v6f};
    const armarx::Matrix_3_6_f m36f{v6f, v6f, v6f};
    const armarx::Matrix_4_6_f m46f{v6f, v6f, v6f, v6f};
    const armarx::Matrix_5_6_f m56f{v6f, v6f, v6f, v6f, v6f};
    const armarx::Matrix_6_6_f m66f{v6f, v6f, v6f, v6f, v6f, v6f};
    BOOST_CHECK_EQUAL((armarx::ToEigen(v1f)), (::Eigen::Matrix<::Ice::Float, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v2f)), (::Eigen::Matrix<::Ice::Float, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v3f)), (::Eigen::Matrix<::Ice::Float, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v4f)), (::Eigen::Matrix<::Ice::Float, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v5f)), (::Eigen::Matrix<::Ice::Float, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v6f)), (::Eigen::Matrix<::Ice::Float, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m11f)), (::Eigen::Matrix<::Ice::Float, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m21f)), (::Eigen::Matrix<::Ice::Float, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m31f)), (::Eigen::Matrix<::Ice::Float, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m41f)), (::Eigen::Matrix<::Ice::Float, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m51f)), (::Eigen::Matrix<::Ice::Float, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m61f)), (::Eigen::Matrix<::Ice::Float, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m12f)), (::Eigen::Matrix<::Ice::Float, 1, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m22f)), (::Eigen::Matrix<::Ice::Float, 2, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m32f)), (::Eigen::Matrix<::Ice::Float, 3, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m42f)), (::Eigen::Matrix<::Ice::Float, 4, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m52f)), (::Eigen::Matrix<::Ice::Float, 5, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m62f)), (::Eigen::Matrix<::Ice::Float, 6, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m13f)), (::Eigen::Matrix<::Ice::Float, 1, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m23f)), (::Eigen::Matrix<::Ice::Float, 2, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m33f)), (::Eigen::Matrix<::Ice::Float, 3, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m43f)), (::Eigen::Matrix<::Ice::Float, 4, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m53f)), (::Eigen::Matrix<::Ice::Float, 5, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m63f)), (::Eigen::Matrix<::Ice::Float, 6, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m14f)), (::Eigen::Matrix<::Ice::Float, 1, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m24f)), (::Eigen::Matrix<::Ice::Float, 2, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m34f)), (::Eigen::Matrix<::Ice::Float, 3, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m44f)), (::Eigen::Matrix<::Ice::Float, 4, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m54f)), (::Eigen::Matrix<::Ice::Float, 5, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m64f)), (::Eigen::Matrix<::Ice::Float, 6, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m15f)), (::Eigen::Matrix<::Ice::Float, 1, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m25f)), (::Eigen::Matrix<::Ice::Float, 2, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m35f)), (::Eigen::Matrix<::Ice::Float, 3, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m45f)), (::Eigen::Matrix<::Ice::Float, 4, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m55f)), (::Eigen::Matrix<::Ice::Float, 5, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m65f)), (::Eigen::Matrix<::Ice::Float, 6, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m16f)), (::Eigen::Matrix<::Ice::Float, 1, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m26f)), (::Eigen::Matrix<::Ice::Float, 2, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m36f)), (::Eigen::Matrix<::Ice::Float, 3, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m46f)), (::Eigen::Matrix<::Ice::Float, 4, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m56f)), (::Eigen::Matrix<::Ice::Float, 5, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m66f)), (::Eigen::Matrix<::Ice::Float, 6, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 1, 1>::Ones() * 3)),
                      v1f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 2, 1>::Ones() * 3)),
                      v2f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 3, 1>::Ones() * 3)),
                      v3f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 4, 1>::Ones() * 3)),
                      v4f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 5, 1>::Ones() * 3)),
                      v5f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 6, 1>::Ones() * 3)),
                      v6f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 1, 2>::Ones() * 3)),
                      m12f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 2, 2>::Ones() * 3)),
                      m22f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 3, 2>::Ones() * 3)),
                      m32f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 4, 2>::Ones() * 3)),
                      m42f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 5, 2>::Ones() * 3)),
                      m52f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 6, 2>::Ones() * 3)),
                      m62f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 1, 3>::Ones() * 3)),
                      m13f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 2, 3>::Ones() * 3)),
                      m23f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 3, 3>::Ones() * 3)),
                      m33f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 4, 3>::Ones() * 3)),
                      m43f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 5, 3>::Ones() * 3)),
                      m53f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 6, 3>::Ones() * 3)),
                      m63f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 1, 4>::Ones() * 3)),
                      m14f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 2, 4>::Ones() * 3)),
                      m24f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 3, 4>::Ones() * 3)),
                      m34f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 4, 4>::Ones() * 3)),
                      m44f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 5, 4>::Ones() * 3)),
                      m54f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 6, 4>::Ones() * 3)),
                      m64f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 1, 5>::Ones() * 3)),
                      m15f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 2, 5>::Ones() * 3)),
                      m25f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 3, 5>::Ones() * 3)),
                      m35f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 4, 5>::Ones() * 3)),
                      m45f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 5, 5>::Ones() * 3)),
                      m55f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 6, 5>::Ones() * 3)),
                      m65f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 1, 6>::Ones() * 3)),
                      m16f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 2, 6>::Ones() * 3)),
                      m26f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 3, 6>::Ones() * 3)),
                      m36f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 4, 6>::Ones() * 3)),
                      m46f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 5, 6>::Ones() * 3)),
                      m56f);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Float, 6, 6>::Ones() * 3)),
                      m66f);
    static_assert(std::is_same<armarx::Vector1d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 1, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector2d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 2, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector3d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 3, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector4d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 4, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector5d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 5, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Vector6d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 6, 1>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_2_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 1, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_2_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 2, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_2_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 3, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_2_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 4, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_2_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 5, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_2_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 6, 2>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_3_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 1, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_3_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 2, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_3_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 3, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_3_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 4, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_3_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 5, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_3_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 6, 3>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_4_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 1, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_4_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 2, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_4_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 3, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_4_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 4, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_4_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 5, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_4_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 6, 4>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_5_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 1, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_5_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 2, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_5_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 3, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_5_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 4, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_5_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 5, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_5_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 6, 5>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_1_6_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 1, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_2_6_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 2, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_3_6_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 3, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_4_6_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 4, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_5_6_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 5, 6>()))>::value,
                  "");
    static_assert(std::is_same<armarx::Matrix_6_6_d,
                               decltype(armarx::ToBasicVectorType(
                                   ::Eigen::Matrix<::Ice::Double, 6, 6>()))>::value,
                  "");
    const armarx::Vector1d v1d{3};
    const armarx::Vector2d v2d{3, 3};
    const armarx::Vector3d v3d{3, 3, 3};
    const armarx::Vector4d v4d{3, 3, 3, 3};
    const armarx::Vector5d v5d{3, 3, 3, 3, 3};
    const armarx::Vector6d v6d{3, 3, 3, 3, 3, 3};
    const armarx::Matrix_1_1_d m11d{v1d};
    const armarx::Matrix_2_1_d m21d{v1d, v1d};
    const armarx::Matrix_3_1_d m31d{v1d, v1d, v1d};
    const armarx::Matrix_4_1_d m41d{v1d, v1d, v1d, v1d};
    const armarx::Matrix_5_1_d m51d{v1d, v1d, v1d, v1d, v1d};
    const armarx::Matrix_6_1_d m61d{v1d, v1d, v1d, v1d, v1d, v1d};
    const armarx::Matrix_1_2_d m12d{v2d};
    const armarx::Matrix_2_2_d m22d{v2d, v2d};
    const armarx::Matrix_3_2_d m32d{v2d, v2d, v2d};
    const armarx::Matrix_4_2_d m42d{v2d, v2d, v2d, v2d};
    const armarx::Matrix_5_2_d m52d{v2d, v2d, v2d, v2d, v2d};
    const armarx::Matrix_6_2_d m62d{v2d, v2d, v2d, v2d, v2d, v2d};
    const armarx::Matrix_1_3_d m13d{v3d};
    const armarx::Matrix_2_3_d m23d{v3d, v3d};
    const armarx::Matrix_3_3_d m33d{v3d, v3d, v3d};
    const armarx::Matrix_4_3_d m43d{v3d, v3d, v3d, v3d};
    const armarx::Matrix_5_3_d m53d{v3d, v3d, v3d, v3d, v3d};
    const armarx::Matrix_6_3_d m63d{v3d, v3d, v3d, v3d, v3d, v3d};
    const armarx::Matrix_1_4_d m14d{v4d};
    const armarx::Matrix_2_4_d m24d{v4d, v4d};
    const armarx::Matrix_3_4_d m34d{v4d, v4d, v4d};
    const armarx::Matrix_4_4_d m44d{v4d, v4d, v4d, v4d};
    const armarx::Matrix_5_4_d m54d{v4d, v4d, v4d, v4d, v4d};
    const armarx::Matrix_6_4_d m64d{v4d, v4d, v4d, v4d, v4d, v4d};
    const armarx::Matrix_1_5_d m15d{v5d};
    const armarx::Matrix_2_5_d m25d{v5d, v5d};
    const armarx::Matrix_3_5_d m35d{v5d, v5d, v5d};
    const armarx::Matrix_4_5_d m45d{v5d, v5d, v5d, v5d};
    const armarx::Matrix_5_5_d m55d{v5d, v5d, v5d, v5d, v5d};
    const armarx::Matrix_6_5_d m65d{v5d, v5d, v5d, v5d, v5d, v5d};
    const armarx::Matrix_1_6_d m16d{v6d};
    const armarx::Matrix_2_6_d m26d{v6d, v6d};
    const armarx::Matrix_3_6_d m36d{v6d, v6d, v6d};
    const armarx::Matrix_4_6_d m46d{v6d, v6d, v6d, v6d};
    const armarx::Matrix_5_6_d m56d{v6d, v6d, v6d, v6d, v6d};
    const armarx::Matrix_6_6_d m66d{v6d, v6d, v6d, v6d, v6d, v6d};
    BOOST_CHECK_EQUAL((armarx::ToEigen(v1d)), (::Eigen::Matrix<::Ice::Double, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v2d)), (::Eigen::Matrix<::Ice::Double, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v3d)), (::Eigen::Matrix<::Ice::Double, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v4d)), (::Eigen::Matrix<::Ice::Double, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v5d)), (::Eigen::Matrix<::Ice::Double, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(v6d)), (::Eigen::Matrix<::Ice::Double, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m11d)), (::Eigen::Matrix<::Ice::Double, 1, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m21d)), (::Eigen::Matrix<::Ice::Double, 2, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m31d)), (::Eigen::Matrix<::Ice::Double, 3, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m41d)), (::Eigen::Matrix<::Ice::Double, 4, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m51d)), (::Eigen::Matrix<::Ice::Double, 5, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m61d)), (::Eigen::Matrix<::Ice::Double, 6, 1>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m12d)), (::Eigen::Matrix<::Ice::Double, 1, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m22d)), (::Eigen::Matrix<::Ice::Double, 2, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m32d)), (::Eigen::Matrix<::Ice::Double, 3, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m42d)), (::Eigen::Matrix<::Ice::Double, 4, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m52d)), (::Eigen::Matrix<::Ice::Double, 5, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m62d)), (::Eigen::Matrix<::Ice::Double, 6, 2>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m13d)), (::Eigen::Matrix<::Ice::Double, 1, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m23d)), (::Eigen::Matrix<::Ice::Double, 2, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m33d)), (::Eigen::Matrix<::Ice::Double, 3, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m43d)), (::Eigen::Matrix<::Ice::Double, 4, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m53d)), (::Eigen::Matrix<::Ice::Double, 5, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m63d)), (::Eigen::Matrix<::Ice::Double, 6, 3>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m14d)), (::Eigen::Matrix<::Ice::Double, 1, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m24d)), (::Eigen::Matrix<::Ice::Double, 2, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m34d)), (::Eigen::Matrix<::Ice::Double, 3, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m44d)), (::Eigen::Matrix<::Ice::Double, 4, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m54d)), (::Eigen::Matrix<::Ice::Double, 5, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m64d)), (::Eigen::Matrix<::Ice::Double, 6, 4>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m15d)), (::Eigen::Matrix<::Ice::Double, 1, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m25d)), (::Eigen::Matrix<::Ice::Double, 2, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m35d)), (::Eigen::Matrix<::Ice::Double, 3, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m45d)), (::Eigen::Matrix<::Ice::Double, 4, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m55d)), (::Eigen::Matrix<::Ice::Double, 5, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m65d)), (::Eigen::Matrix<::Ice::Double, 6, 5>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m16d)), (::Eigen::Matrix<::Ice::Double, 1, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m26d)), (::Eigen::Matrix<::Ice::Double, 2, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m36d)), (::Eigen::Matrix<::Ice::Double, 3, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m46d)), (::Eigen::Matrix<::Ice::Double, 4, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m56d)), (::Eigen::Matrix<::Ice::Double, 5, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToEigen(m66d)), (::Eigen::Matrix<::Ice::Double, 6, 6>::Ones() * 3));
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 1, 1>::Ones() * 3)),
                      v1d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 2, 1>::Ones() * 3)),
                      v2d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 3, 1>::Ones() * 3)),
                      v3d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 4, 1>::Ones() * 3)),
                      v4d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 5, 1>::Ones() * 3)),
                      v5d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 6, 1>::Ones() * 3)),
                      v6d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 1, 2>::Ones() * 3)),
                      m12d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 2, 2>::Ones() * 3)),
                      m22d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 3, 2>::Ones() * 3)),
                      m32d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 4, 2>::Ones() * 3)),
                      m42d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 5, 2>::Ones() * 3)),
                      m52d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 6, 2>::Ones() * 3)),
                      m62d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 1, 3>::Ones() * 3)),
                      m13d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 2, 3>::Ones() * 3)),
                      m23d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 3, 3>::Ones() * 3)),
                      m33d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 4, 3>::Ones() * 3)),
                      m43d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 5, 3>::Ones() * 3)),
                      m53d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 6, 3>::Ones() * 3)),
                      m63d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 1, 4>::Ones() * 3)),
                      m14d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 2, 4>::Ones() * 3)),
                      m24d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 3, 4>::Ones() * 3)),
                      m34d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 4, 4>::Ones() * 3)),
                      m44d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 5, 4>::Ones() * 3)),
                      m54d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 6, 4>::Ones() * 3)),
                      m64d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 1, 5>::Ones() * 3)),
                      m15d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 2, 5>::Ones() * 3)),
                      m25d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 3, 5>::Ones() * 3)),
                      m35d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 4, 5>::Ones() * 3)),
                      m45d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 5, 5>::Ones() * 3)),
                      m55d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 6, 5>::Ones() * 3)),
                      m65d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 1, 6>::Ones() * 3)),
                      m16d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 2, 6>::Ones() * 3)),
                      m26d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 3, 6>::Ones() * 3)),
                      m36d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 4, 6>::Ones() * 3)),
                      m46d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 5, 6>::Ones() * 3)),
                      m56d);
    BOOST_CHECK_EQUAL((armarx::ToBasicVectorType(::Eigen::Matrix<::Ice::Double, 6, 6>::Ones() * 3)),
                      m66d);
}

BOOST_AUTO_TEST_CASE(ToBasicVectorType_vector_convert_Test)
{
    static_assert(std::is_same<std::vector<armarx::Vector1byte>,
                               decltype(armarx::ToBasicVectorType(
                                   std::vector<::Eigen::Matrix<::Ice::Byte, 1, 1>>()))>::value,
                  "");
    static_assert(
        std::is_same<std::vector<::Eigen::Matrix<::Ice::Byte, 1, 1>>,
                     decltype(armarx::ToEigen(std::vector<armarx::Vector1byte>()))>::value,
        "");
}
