/*
* This file is part of ArmarX.
*
* Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarXCore::core::Test
* @author     Nils Adermann (naderman at naderman dot de)
* @date       2010
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/
#pragma once

#include <cstdlib>
#include <map>
#include <memory>
#include <random>
#include <stdexcept>

#include <Ice/Ice.h>
#include <IceGrid/Registry.h>

#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/test/TestArmarXManager.h>
#include <ArmarXCore/util/CPPUtility/filesystem.h>

/**
* Helper class IceTestHelper using Ice in tests
*/
class IceTestHelper : public IceUtil::Shared
{
public:
    IceTestHelper(int registryPort = 11220, int port = 11221);

    ~IceTestHelper() override;

    void startEnvironment();

    void startIceStorm();

    void startIceGrid();

    void stopIceGrid();

    void deployApplication(const std::string& xmlPath);

    Ice::ObjectPrx registerAdapter(const std::string& name, Ice::ObjectPtr object);

    Ice::CommunicatorPtr getCommunicator();

    /**
     * Retrieves a proxy object.
     *
     * @param  objectIdentity The object's identity, e.g. Log
     *
     * @return A proxy of the remote instance.
     */
    template <class ProxyType>
    ProxyType
    getProxy(const std::string& name)
    {
        std::string query;
        if (endpoints.count(name) > 0)
        {
            query = name + std::string(":") + endpoints[name];
        }
        else
        {
            query = name;
        }
        Ice::ObjectPrx base = ic->stringToProxy(query);
        ProxyType proxy = ProxyType::checkedCast(base);

        if (!proxy)
        {
            throw std::runtime_error("Invalid proxy");
        }

        return proxy;
    }

    Ice::StringSeq getArgs() const;

protected:
    void init();

    int exec(const std::string& command);

private:
    Ice::StringSeq args;
    bool grid;
    std::string configDir;
    Ice::CommunicatorPtr ic;
    //        armarx::IceGridAdminPtr admin;
    std::map<std::string, std::string> endpoints;

    int registryPort;
    std::string registryHost = "0.0.0.0";
    int port;
};

using IceTestHelperPtr = IceUtil::Handle<IceTestHelper>;
