/*
* This file is part of ArmarX.
*
* Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarXCore::core::Test
* @author     Nils Adermann (naderman at naderman dot de)
* @date       2010
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#define BOOST_TEST_MODULE ArmarX::Core::TextArmarXManagerTest
#define ARMARX_BOOST_TEST

#include <ArmarXCore/Test.h>

#include <IceStorm/IceStorm.h>

#include <ArmarXCore/core/test/IceTestHelper.h>
#include <ArmarXCore/core/test/TestArmarXManager.h>
#include <ArmarXCore/core/time/Clock.h>

using namespace armarx;

BOOST_AUTO_TEST_CASE(test_TestArmarXManager)
{
    IceTestHelper iceTestHelper;
    iceTestHelper.startEnvironment();

    Ice::CommunicatorPtr communicator = iceTestHelper.getCommunicator();
    BOOST_CHECK(communicator);

    bool oneSuccess = false;

    std::vector<std::string> endpointVersions{"1.1", "1.0"};
    for (const std::string& ev : endpointVersions)
    {
        BOOST_TEST_CONTEXT("Endpoint version: " << ev)
        {
            std::stringstream topicManagerID;
            topicManagerID << "IceStorm/TopicManager -t -e " << ev
                // These must not be specified.
                // << ":tcp"
                // << " -p " + std::to_string(iceTestHelper.registryPort)
                // << " -h " + iceTestHelper.registryHost
                ;
            Ice::ObjectPrx proxy;
            BOOST_CHECK_NO_THROW(proxy = communicator->stringToProxy(topicManagerID.str()));
            BOOST_CHECK(proxy);
            BOOST_TEST_MESSAGE("Name: '" << proxy->ice_getIdentity().name << "'");
            BOOST_TEST_MESSAGE("Category: '" << proxy->ice_getIdentity().category << "'");
            try
            {
                proxy->ice_ping();
            }
            catch (const ::Ice::NoEndpointException& e)
            {
                BOOST_TEST_INFO("Caught exception: " << e.what());
                // In CI:
                // ::Ice::NoEndpointException:
                // no suitable endpoint available for proxy `IceStorm/TopicManager -t -e 1.0
                BOOST_CHECK(false);
            }
            // BOOST_REQUIRE_NO_THROW();
            BOOST_CHECK_NO_THROW(IceStorm::TopicManagerPrx::checkedCast(proxy));

            if (IceStorm::TopicManagerPrx::checkedCast(proxy))
            {
                oneSuccess = true;
            }
        }
    }

    BOOST_REQUIRE(oneSuccess);

    BOOST_CHECK_NO_THROW(IceManager::GetTopicManager(communicator));

    Clock::WaitFor(armarx::Duration::SecondsDouble(5));

    // This MUST be a Ptr - otherwise the test will never finish.
    TestArmarXManagerPtr manager;
    BOOST_CHECK_NO_THROW(
        manager = new TestArmarXManager("TextArmarXManagerTest", iceTestHelper.getCommunicator()));

    BOOST_REQUIRE(manager);
    BOOST_CHECK(not manager->isShutdown());

    Clock::WaitFor(armarx::Duration::SecondsDouble(0.5));

    manager->shutdown();

    BOOST_CHECK(manager->isShutdown());
}
