/*
* This file is part of ArmarX.
*
* Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarX::Core
* @author     Kai Welke <welke@kit.edu>
* @copyright  2011 Humanoids Group, HIS, KIT
* @license    http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

#include <ArmarXCore/interface/observers/ConditionCheckBase.ice>
#include <ArmarXCore/interface/observers/Event.ice>
#include <ArmarXCore/interface/observers/ObserverInterface.ice>
#include <ArmarXCore/interface/observers/TermImplBase.ice>
#include <ArmarXCore/interface/observers/VariantBase.ice>

module armarx
{

    dictionary<int, ConditionRootBase> ConditionRegistry;

    interface ConditionHandlerInterface
    {
        // condition installation and removal
        ConditionIdentifier installCondition(EventListenerInterface * listener,
                                             TermImplBase term,
                                             EventBase e,
                                             bool onlyFireOnce,
                                             bool reportDatafields,
                                             DatafieldRefList additionalReportDatafields)
            throws InvalidConditionException;
        ConditionIdentifier installConditionWithDescription(
            EventListenerInterface * listener,
            TermImplBase term,
            EventBase e,
            string desc,
            bool onlyFireOnce,
            bool reportDatafields,
            DatafieldRefList additionalReportDatafields) throws InvalidConditionException;

        void removeCondition(ConditionIdentifier id);
        void removeAllConditions();
        Ice::StringSeq getObserverNames();
        ConditionRegistry getActiveConditions();
        ConditionRegistry getPastConditions();
        ConditionRootBase getCondition(int id);
    };
};
