/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package
 * @author
 * @date
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include "CppCtor.h"
#include "CppEnum.h"
#include "CppField.h"
#include "CppMethod.h"
#include "MetaClass.h"

namespace armarx
{
    class CppClass;
    using CppClassPtr = std::shared_ptr<CppClass>;

    class CppClass : virtual public MetaClass
    {
    public:
        CppClass(const std::vector<std::string>& namespaces,
                 const std::string& name,
                 const std::string& templates = "");

        virtual void write(const MetaWriterPtr& writer) override;
        void writeCpp(const CppWriterPtr& writer);
        static void WriteCpp(const std::vector<CppClassPtr>& classes, const CppWriterPtr& writer);

        void addPublicMethod(const CppMethodPtr method);
        CppMethodPtr addPublicMethod(const std::string& header,
                                     const std::string& doc = "",
                                     const bool enforceBlockGeneration = false);

        void addCtor(const CppCtorPtr&);
        CppCtorPtr addCtor(const std::string& arguments);
        CppCtorPtr addCtor(const std::string& arguments, const CppBlockPtr& b);

        void addPublicField(const CppFieldPtr& field);
        void addPrivateField(const CppFieldPtr& field);
        void addProtectedField(const CppFieldPtr& field);

        void addInnerClass(const CppClassPtr inner);
        CppClassPtr addInnerClass(const std::string& name);

        void addInnerEnum(const CppEnumPtr inner);
        CppEnumPtr addInnerEnum(const std::string& name);

        void addInherit(const std::string& inherit);

        void addInclude(const std::string include);
        bool hasInclude(const std::string include);

        void setTemplates(const std::string&);


    private:
        std::vector<std::string> namespaces;
        std::string templates;
        std::vector<CppMethodPtr> publicMethods;
        std::vector<CppFieldPtr> privateFields;
        std::vector<CppFieldPtr> protectedFields;
        std::vector<CppFieldPtr> publicFields;
        std::vector<std::string> inherit;
        std::vector<CppEnumPtr> innerEnums;
        std::vector<CppClassPtr> innerClasses;
        std::vector<CppCtorPtr> ctors;
        std::vector<std::string> includes;
    };
} // namespace armarx
