/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package
 * @author
 * @date
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#include "CppCtor.h"

#include <boost/format.hpp>

#include <SimoxUtility/algorithm/string/string_tools.h>

#include <ArmarXCore/core/exceptions/LocalException.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

namespace armarx
{
    CppCtor::CppCtor(const std::string& header)
    {
        this->header = header;
    }

    CppCtor::CppCtor(const std::string& header, const CppBlockPtr& b)
    {
        this->header = header;
        this->block = b;
    }

    void
    CppCtor::addInitListEntries(const std::vector<std::pair<std::string, std::string>>& list)
    {
        for (const auto& [target, expr] : list)
        {
            addInitListEntry(target, expr);
        }
    }

    void
    CppCtor::addInitListEntry(const std::string& target, const std::string& expr)
    {
        this->initList.push_back(boost::str(boost::format("%s(%s)") % target % expr));
    }

    void
    CppCtor::addLine(const std::string& line)
    {
        if (!block)
        {
            this->block.reset(new CppBlock());
        }
        block->addLine(line);
    }

    void
    CppCtor::addLine(const boost::basic_format<char>& line)
    {
        if (!block)
        {
            this->block.reset(new CppBlock());
        }
        block->addLine(line);
    }

    void
    CppCtor::setBlock(const CppBlockPtr& b)
    {
        block = b;
    }

    CppBlockPtr
    CppCtor::getBlock() const
    {
        return block;
    }

    void
    CppCtor::writeCpp(const CppWriterPtr& writer)
    {
        writer->body.line(header);

        if (simox::alg::ends_with(simox::alg::trim_copy(header), ";")) // its a protoype
        {
            ARMARX_CHECK(initList.empty());
            ARMARX_CHECK(!enforceBlockGeneration);
            ARMARX_CHECK(!block);
            return;
        }

        for (size_t i = 0; i < initList.size(); i++)
        {
            writer->body.line(boost::str(boost::format("%s%s%s") % (i == 0 ? ": " : "  ") %
                                         initList.at(i) % (i < initList.size() - 1 ? "," : "")),
                              1);
        }

        auto writeBlock = [this, writer]()
        {
            if (block)
            {
                block->writeCpp(writer);
            }
            else
            {
                CppBlockPtr b = std::make_shared<CppBlock>();
                b->writeCpp(writer);
            }
        };

        //if (enforceBlockGeneration)
        //{
        //    writeBlock();
        //}
        //else if (block || initList.size() > 0)
        //{
        writeBlock();
        //}
    }

    void
    CppCtor::setEnforceBlockGeneration(const bool enforceBlockGeneration)
    {
        this->enforceBlockGeneration = enforceBlockGeneration;
    }
} // namespace armarx
