/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package
 * @author
 * @date
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#include "CppMethod.h"

#include <boost/format.hpp>

#include <SimoxUtility/algorithm/string/string_tools.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>
#include <ArmarXCore/core/logging/Logging.h>

using namespace armarx;

CppMethod::CppMethod(const std::string& header,
                     const std::string& doc,
                     const bool enforceBlockGeneration) :
    header(header), // may be suffixed with ;
    doc(doc),
    enforceBlockGeneration(enforceBlockGeneration)
{
}

void
CppMethod::writeCpp(const CppWriterPtr& writer)
{
    if (!doc.empty())
    {
        std::string delimiters = "\n";
        std::vector<std::string> doclines = simox::alg::split(doc, delimiters);
        writer->body.line("/**");

        for (auto& line : doclines)
        {
            writer->body.line(" * " + line);
        }

        writer->body.line(" */");
    }

    writer->body.line(header);

    if (simox::alg::ends_with(simox::alg::trim_copy(header), ";"))
    {
        ARMARX_CHECK(!enforceBlockGeneration);
        ARMARX_CHECK(!block);
        return;
    }

    auto writeBlock = [this, writer]()
    {
        if (block)
        {
            block->writeCpp(writer);
        }
        else
        {
            CppBlockPtr b = std::make_shared<CppBlock>();
            b->writeCpp(writer);
        }
    };

    //if (enforceBlockGeneration)
    //{
    writeBlock();
    //}
    //else if (block)
    //{
    //    block->writeCpp(writer);
    //}
}

void
CppMethod::addLine(const std::string& line)
{
    if (!block)
    {
        this->block.reset(new CppBlock());
    }
    block->addLine(line);
}

void
CppMethod::addLine(const boost::basic_format<char>& line)
{
    if (!block)
    {
        this->block.reset(new CppBlock());
    }
    block->addLine(line);
}

void
CppMethod::addBlock(const CppBlockPtr& new_block)
{
    if (!block)
    {
        this->block.reset(new CppBlock());
    }
    block->addBlock(new_block);
}

void
CppMethod::setBlock(const CppBlockPtr& new_block)
{
    block = new_block;
}

void
CppMethod::setCompact(const bool compact)
{
    this->compact = compact;
}

void
CppMethod::setEnforceBlockGeneration(const bool enforceBlockGeneration)
{
    this->enforceBlockGeneration = enforceBlockGeneration;
}
