/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package
 * @author
 * @date
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <memory>
#include <sstream>
#include <string>
#include <vector>

// Simox
#include <SimoxUtility/algorithm/string.h>

namespace armarx
{
    class MetaWriter;
    using MetaWriterPtr = std::shared_ptr<MetaWriter>;

    class Lines
    {
    public:
        enum class LineType
        {
            StartBlock,
            EndBlock,
            Normal,
            Empty
        };

        std::string name;
        std::stringstream ss;
        std::vector<std::string> lines;
        int indent = 0;

        Lines() = delete;
        Lines(const std::string&, const std::string& = "    ", const LineType = LineType::Empty);

        void startBlock();
        void startBlock(const std::string& line);
        void endBlock();
        void endBlock(const std::string& line);
        void endBlockComment(std::string comment);

        void line();
        void line(const std::string& line, int indentDelta = 0);
        void lineCheckDuplicate(const std::string& line, int indentDelta = 0);
        void enforceEmptyLine();

        Lines::LineType getLastLineType();

    private:
        void lineInternal(const std::string& line, int indent);

    private:
        std::string indentChars;
        LineType lastLineType;
    };

    class MetaWriter
    {
    public:
        MetaWriter(const std::string& doc);
        virtual ~MetaWriter() = default;

        void lineHeader();
        void lineFooter();

        std::string getString();

        static void ConvertToTextWithMaxWidth(unsigned int maxTextWidth,
                                              const std::vector<std::string>& docLines,
                                              std::vector<std::string>& linesOut);

    public:
        Lines header;
        Lines body;
        Lines footer;

    protected:
        std::string fileDoc;
    };
} // namespace armarx
