/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2017, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarX
 * @author     Mirko Waechter( mirko.waechter at kit dot edu)
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#include "AverageFilter.h"

namespace armarx::rtfilters
{

    AverageFilter::AverageFilter(size_t windowSize) : RTFilterBase(windowSize)
    {
    }

    RTFilterBasePtr
    AverageFilter::clone() const
    {
        RTFilterBase* v = new AverageFilter(*this);
        return RTFilterBasePtr(v);
    }

    double
    AverageFilter::calculate()
    {
        if (dataHistory.empty())
        {
            return 0.0;
        }

        double sum = 0.0f;
        for (auto it = dataHistory.begin(); it != dataHistory.end(); it++)
        {
            sum += it->second;
        }
        double result = sum / dataHistory.size();
        return result;
    }

} // namespace armarx::rtfilters
