/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2011-2016, High Performance Humanoid Technologies (H2T), Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::units
 * @author     Peter Kaiser (peter dot kaiser at kit dot edu)
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/application/properties/Properties.h>
#include <ArmarXCore/interface/observers/ExampleUnitInterface.h>
#include <ArmarXCore/interface/observers/ExampleUnitObserverInterface.h>
#include <ArmarXCore/observers/Observer.h>

namespace armarx
{
    class ExampleUnitObserverPropertyDefinitions : public ObserverPropertyDefinitions
    {
    public:
        ExampleUnitObserverPropertyDefinitions(std::string prefix) :
            ObserverPropertyDefinitions(prefix)
        {
            // No required properties
        }
    };

    //! [ObserversDocumentation ObserverComponent1]
    class ARMARXCORE_IMPORT_EXPORT ExampleUnitObserver :
        virtual public Observer,
        virtual public ExampleUnitObserverInterface
    {
    public:
        ExampleUnitObserver();

        std::string
        getDefaultName() const override
        {
            return "ExampleUnitObserver";
        }

        void onInitObserver() override;
        void onConnectObserver() override;

        void reportPeriodicValue(const VariantBasePtr& value,
                                 Ice::Double timestamp,
                                 const ::Ice::Current& = Ice::emptyCurrent) override;

        PropertyDefinitionsPtr createPropertyDefinitions() override;
        using Observer::removeDatafield;
    };

    using ExampleUnitObserverPtr = IceInternal::Handle<ExampleUnitObserver>;
    //! [ObserversDocumentation ObserverComponent1]
} // namespace armarx
