#include "CMakeSourceListGenerator.h"

namespace armarx
{

    CMakeSourceListGenerator::CMakeSourceListGenerator()
    {
    }

    std::string
    CMakeSourceListGenerator::GenerateCMakeFile(const std::string& groupName,
                                                const std::vector<std::string>& stateXMLFiles,
                                                const std::vector<std::string>& stateSourceFiles,
                                                const std::vector<std::string>& stateHeaderFiles,
                                                const std::vector<std::string>& libs)
    {

        CMakeWriter writer;
        writer.line("SET(HEADERS ${HEADERS}");
        for (auto& file : stateXMLFiles)
        {
            writer.line(file, 1);
        }
        writer.line(")", 1);
        writer.line();
        writer.line("SET(HEADERS ${HEADERS}");
        for (auto& file : stateHeaderFiles)
        {
            writer.line(file, 1);
        }
        writer.line(")", 1);
        writer.line();
        writer.line("SET(SOURCES ${SOURCES}");
        for (auto& file : stateSourceFiles)
        {
            writer.line(file, 1);
        }
        writer.line(")", 1);

        writer.line();
        writer.line("SET(COMPONENT_LIBS ${COMPONENT_LIBS}");
        for (auto& file : libs)
        {
            writer.line(file, 1);
        }
        writer.line(")", 1);


        return writer.getString();
    }

    std::string
    CMakeSourceListGenerator::GenerateCMakeFileNextGen(
        const std::string& groupName,
        const std::vector<std::string>& stateXMLFiles,
        const std::vector<std::string>& stateSourceFiles,
        const std::vector<std::string>& stateHeaderFiles,
        const std::vector<std::string>& libs)
    {
        CMakeWriter writer;

        writer.line("armarx_add_statechart_library(" + groupName + "_statechart");

        // headers
        if (not stateHeaderFiles.empty())
        {
            writer.line("HEADERS", 1);
            //writer.line("# autogenerated headers", 2);
            for (const auto& file : stateHeaderFiles)
            {
                writer.line(file, 2);
            }
        }

        writer.line("STATE_FILES", 1);
        for (const auto& file : stateXMLFiles)
        {
            writer.line(file, 2);
        }

        // sources
        if (not stateSourceFiles.empty())
        {
            writer.line("SOURCES", 1);
            for (const auto& file : stateSourceFiles)
            {
                writer.line(file, 2);
            }
        }

        // dependencies
        writer.line("DEPENDENCIES", 1);
        for (const auto& file : libs)
        {
            writer.line(file, 2);
        }

        // if no statechart cpp files are added, generate an interface library with the dependencies attached
        if (stateSourceFiles.empty())
        {
            writer.line("INTERFACE", 1);
        }
        // if statechart cpp files are available, generate an object library with the dependencies within the public scope
        else
        {
            writer.line("OBJECT", 1);
        }
        writer.line(")", 0);

        return writer.getString();
    }

} // namespace armarx
